@@ -9,13 +9,17 @@
 #include "FlatpakBackend.h"
 #include "FlatpakFetchDataJob.h"
 #include "FlatpakSourcesBackend.h"
+#include "LazyIconResolver.h"
 #include "config-paths.h"
+#include "libdiscover_backend_flatpak_debug.h"
 
 #include <Transaction/AddonList.h>
 
+#include <AppStreamQt/developer.h>
 #include <AppStreamQt/icon.h>
 #include <AppStreamQt/screenshot.h>
 #include <AppStreamQt/utils.h>
+#include <AppStreamQt/version.h>
 #include <appstream/AppStreamUtils.h>
 
 #include <KConfigGroup>
@@ -25,10 +29,11 @@
 #include <KLocalizedString>
 
 #include <AppStreamQt/release.h>
-#include <QDebug>
+#include <QCoroCore>
 #include <QDesktopServices>
 #include <QDir>
 #include <QDirIterator>
+#include <QEvent>
 #include <QFileInfo>
 #include <QFutureWatcher>
 #include <QIcon>
@@ -36,25 +41,29 @@
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QProcess>
+#include <QQueue>
 #include <QStringList>
 #include <QTemporaryFile>
 #include <QTimer>
 #include <QUrlQuery>
 #include <QtConcurrentRun>
 
+using namespace Qt::StringLiterals;
+
 static QString iconCachePath(const AppStream::Icon &icon)
 {
     Q_ASSERT(icon.kind() == AppStream::Icon::KindRemote);
     return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
-const QStringList FlatpakResource::s_objects({
+const QStringList FlatpakResource::s_topObjects({
     QStringLiteral("qrc:/qml/FlatpakAttention.qml"),
     QStringLiteral("qrc:/qml/FlatpakRemoveData.qml"),
     QStringLiteral("qrc:/qml/FlatpakOldBeta.qml"),
     QStringLiteral("qrc:/qml/FlatpakEolReason.qml"),
 });
 const QStringList FlatpakResource::s_bottomObjects({QStringLiteral("qrc:/qml/PermissionsList.qml")});
+
 Q_GLOBAL_STATIC(QNetworkAccessManager, manager)
 
 FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent)
@@ -71,35 +80,54 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
 
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
-    if (icons.count() == 1 && icons.constFirst().kind() == AppStream::Icon::KindRemote) {
-        const auto icon = icons.constFirst();
-        const QString fileName = iconCachePath(icon);
-        if (!QFileInfo::exists(fileName)) {
-            const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
-            // Create $HOME/.cache/discover/icons folder
-            cacheDir.mkdir(QStringLiteral("icons"));
-            auto reply = manager->get(QNetworkRequest(icon.url()));
-            connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
-                if (reply->error() == QNetworkReply::NoError) {
-                    QByteArray iconData = reply->readAll();
-                    QFile file(fileName);
-                    if (file.open(QIODevice::WriteOnly)) {
-                        file.write(iconData);
-                    } else {
-                        qDebug() << "could not find icon for" << packageName() << reply->url();
-                        QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+    m_stockIcon = std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
+        return icon.kind() == AppStream::Icon::KindStock && AppStreamUtils::kIconLoaderHasIcon(icon.name());
+    });
+    connect(this, &AbstractResource::iconChanged, this, [this] {
+        Q_EMIT backend()->resourcesChanged(this, {"icon"});
+    });
+    if (!m_stockIcon && !icons.isEmpty() && !std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
+            return icon.kind() == AppStream::Icon::KindLocal || icon.kind() == AppStream::Icon::KindCached;
+        })) {
+        for (const auto &icon : icons) {
+            if (icon.kind() != AppStream::Icon::KindRemote) {
+                continue;
+            }
+            const QString fileName = iconCachePath(icon);
+            if (!QFileInfo::exists(fileName)) {
+                const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+                // Create $HOME/.cache/discover/icons folder
+                cacheDir.mkdir(QStringLiteral("icons"));
+                auto reply = manager->get(QNetworkRequest(icon.url()));
+                connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
+                    if (reply->error() == QNetworkReply::NoError) {
+                        QByteArray iconData = reply->readAll();
+                        QFile file(fileName);
+                        if (file.open(QIODevice::WriteOnly)) {
+                            file.write(iconData);
+                        } else {
+                            qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not find icon for" << packageName() << reply->url();
+                            QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+                        }
+                        file.close();
+                        Q_EMIT iconChanged();
+                        reply->deleteLater();
                     }
-                    file.close();
-                    Q_EMIT iconChanged();
-                    reply->deleteLater();
-                }
-            });
+                });
+            }
         }
     }
 
     connect(this, &FlatpakResource::stateChanged, this, &FlatpakResource::hasDataChanged);
 }
 
+FlatpakBackend *FlatpakResource::backend() const
+{
+    const auto backend = qobject_cast<FlatpakBackend *>(AbstractResource::backend());
+    Q_ASSERT(backend);
+    return backend;
+}
+
 AppStream::Component FlatpakResource::appstreamComponent() const
 {
     return m_appdata;
@@ -113,7 +141,7 @@ QList<PackageState> FlatpakResource::addonsInformation()
 QString FlatpakResource::availableVersion() const
 {
     if (m_availableVersion.isEmpty()) {
-        const auto releases = m_appdata.releases();
+        const auto releases = m_appdata.releasesPlain().entries();
         if (!releases.isEmpty()) {
             auto latestVersion = releases.constFirst().version();
             for (const auto &release : releases) {
@@ -154,7 +182,6 @@ void FlatpakResource::updateFromRef(FlatpakRef *ref)
 {
     setArch(QString::fromUtf8(flatpak_ref_get_arch(ref)));
     setBranch(QString::fromUtf8(flatpak_ref_get_branch(ref)));
-    setCommit(QString::fromUtf8(flatpak_ref_get_commit(ref)));
     setFlatpakName(QString::fromUtf8(flatpak_ref_get_name(ref)));
     setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp : extends().isEmpty() ? Runtime : Extension);
     setObjectName(packageName());
@@ -166,23 +193,17 @@ void FlatpakResource::updateFromAppStream()
     g_autoptr(GError) localError = nullptr;
     g_autoptr(FlatpakRef) ref = flatpak_ref_parse(refstr.toUtf8().constData(), &localError);
     if (!ref) {
-        qDebug() << "failed to obtain ref" << refstr << localError->message;
+        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "failed to obtain ref" << refstr << localError->message;
         return;
     }
     updateFromRef(ref);
 }
 
-QString FlatpakResource::ref() const
+bool FlatpakResource::hasCategory(const QString &category) const
 {
-    return typeAsString() + QLatin1Char('/') + flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
-}
-
-QStringList FlatpakResource::categories()
-{
-    auto cats = m_appdata.categories();
-    if (m_appdata.kind() != AppStream::Component::KindAddon)
-        cats.append(QStringLiteral("Application"));
-    return cats;
+    if (m_appdata.kind() != AppStream::Component::KindAddon && category == QStringLiteral("Application"))
+        return true;
+    return m_appdata.hasCategory(category);
 }
 
 QString FlatpakResource::comment()
@@ -195,54 +216,51 @@ QString FlatpakResource::comment()
     return QString();
 }
 
-QString FlatpakResource::commit() const
-{
-    return m_commit;
-}
-
 quint64 FlatpakResource::downloadSize() const
 {
     return m_downloadSize;
 }
 
-QVariant FlatpakResource::icon() const
+void FlatpakResource::resolveIcon()
 {
-    QIcon ret;
+    m_icon = QIcon();
     const auto icons = m_appdata.icons();
 
     if (!m_bundledIcon.isNull()) {
-        ret = QIcon(m_bundledIcon);
+        m_icon = QIcon(m_bundledIcon);
     } else if (icons.isEmpty()) {
-        ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
-    } else
+        m_icon = QIcon::fromTheme(QStringLiteral("package-x-generic"));
+    } else {
         for (const AppStream::Icon &icon : icons) {
             switch (icon.kind()) {
             case AppStream::Icon::KindLocal:
             case AppStream::Icon::KindCached: {
                 const QString path = icon.url().toLocalFile();
                 if (QDir::isRelativePath(path)) {
-                    const QString appstreamLocation = installationPath() + "/appstream/" + origin() + '/' + flatpak_get_default_arch() + "/active/icons/";
+                    const QString appstreamLocation =
+                        installationPath() + "/appstream/"_L1 + origin() + '/'_L1 + QString::fromUtf8(flatpak_get_default_arch()) + "/active/icons/"_L1;
                     QDirIterator dit(appstreamLocation, QDirIterator::Subdirectories);
                     while (dit.hasNext()) {
                         const auto currentPath = dit.next();
                         if (dit.fileName() == path) {
-                            ret.addFile(currentPath, icon.size());
+                            m_icon->addFile(currentPath, icon.size());
                         }
                     }
                 } else {
-                    ret.addFile(path, icon.size());
+                    m_icon->addFile(path, icon.size());
                 }
-            } break;
+                break;
+            }
             case AppStream::Icon::KindStock: {
-                const auto ret = QIcon::fromTheme(icon.name());
-                if (!ret.isNull())
-                    return ret;
+                if (m_stockIcon) {
+                    m_icon = QIcon::fromTheme(icon.name());
+                }
                 break;
             }
             case AppStream::Icon::KindRemote: {
                 const QString fileName = iconCachePath(icon);
                 if (QFileInfo::exists(fileName)) {
-                    ret.addFile(fileName, icon.size());
+                    m_icon->addFile(fileName, icon.size());
                 }
                 break;
             }
@@ -250,22 +268,33 @@ QVariant FlatpakResource::icon() const
                 break;
             }
         }
-
-    if (ret.isNull()) {
-        ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
     }
 
-    return ret;
+    if (m_icon->isNull()) {
+        m_icon = QIcon::fromTheme(QStringLiteral("package-x-generic"));
+    }
+
+    Q_EMIT iconChanged();
+}
+
+QVariant FlatpakResource::icon() const
+{
+    if (m_icon.has_value()) {
+        return m_icon.value();
+    }
+
+    LazyIconResolver::instance()->queue(const_cast<FlatpakResource *>(this));
+    return u"package-x-generic"_s;
 }
 
 QString FlatpakResource::installedVersion() const
 {
-    g_autoptr(FlatpakInstalledRef) ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
+    g_autoptr(FlatpakInstalledRef) ref = backend()->getInstalledRefForApp(this);
     if (ref) {
         const char *appdataVersion = flatpak_installed_ref_get_appdata_version(ref);
 
         if (appdataVersion) {
-            return appdataVersion;
+            return QString::fromUtf8(appdataVersion);
         }
     }
     return branch();
@@ -280,7 +309,7 @@ AbstractResource::Type FlatpakResource::type() const
 {
     switch (m_type) {
     case FlatpakResource::Runtime:
-        return Technical;
+        return ApplicationSupport;
     case FlatpakResource::Extension:
         return Addon;
     default:
@@ -388,9 +417,21 @@ QString FlatpakResource::displayOrigin() const
     return !m_displayOrigin.isEmpty() ? m_displayOrigin : m_origin;
 }
 
+// Note: The following three methods are not using each other for optimization purposes:
+// use string builder with arguments directly instead of temporary allocated strings.
 QString FlatpakResource::packageName() const
 {
-    return flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
+    return flatpakName() + '/'_L1 + arch() + '/'_L1 + branch();
+}
+
+QString FlatpakResource::ref() const
+{
+    return typeAsString() + '/'_L1 + flatpakName() + '/'_L1 + arch() + '/'_L1 + branch();
+}
+
+QString FlatpakResource::installPath() const
+{
+    return installationPath() + '/'_L1 + typeAsString() + '/'_L1 + flatpakName() + '/'_L1 + arch() + '/'_L1 + branch() + "/active"_L1;
 }
 
 FlatpakResource::PropertyState FlatpakResource::propertyState(FlatpakResource::PropertyKind kind) const
@@ -424,25 +465,14 @@ quint64 FlatpakResource::size()
 
 QString FlatpakResource::sizeDescription()
 {
-    KFormat f;
-    if (!isInstalled() || canUpgrade()) {
-        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet || propertyState(DownloadSize) == Fetching
-            || propertyState(InstalledSize) == Fetching) {
-            qobject_cast<FlatpakBackend *>(backend())->updateAppSize(this);
-            return i18n("Retrieving size information");
-        } else if (propertyState(DownloadSize) == UnknownOrFailed || propertyState(InstalledSize) == UnknownOrFailed) {
-            return i18n("Unknown size");
-        } else {
-            return i18nc("@info app size", "%1 to download, %2 on disk", f.formatByteSize(downloadSize()), f.formatByteSize(installedSize()));
-        }
+    if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
+        backend()->updateAppSize(this);
+        return i18n("Retrieving size information");
+    } else if (propertyState(InstalledSize) == UnknownOrFailed) {
+        return i18nc("@label app size", "Unknown");
     } else {
-        if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
-            return i18n("Retrieving size information");
-        } else if (propertyState(InstalledSize) == UnknownOrFailed) {
-            return i18n("Unknown size");
-        } else {
-            return i18nc("@info app size", "%1 on disk", f.formatByteSize(installedSize()));
-        }
+        const KFormat f;
+        return f.formatByteSize(installedSize());
     }
 }
 
@@ -456,7 +486,7 @@ FlatpakResource::ResourceType FlatpakResource::resourceType() const
     return m_type;
 }
 
-QString FlatpakResource::typeAsString() const
+QLatin1String FlatpakResource::typeAsString() const
 {
     switch (m_type) {
     case FlatpakResource::Runtime:
@@ -481,18 +511,18 @@ void FlatpakResource::invokeApplication() const
     if (!launchables.isEmpty()) {
         desktopFileName = launchables.constFirst();
     } else {
-        qWarning() << "Failed to find launchable for " << m_appdata.name() << ", using AppStream identifier instead";
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to find launchable for " << m_appdata.name() << ", using AppStream identifier instead";
         desktopFileName = appstreamId();
     }
 
     KService::Ptr service = KService::serviceByStorageId(desktopFileName);
 
     if (!service) {
-        qWarning() << "Failed to find service" << desktopFileName;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to find service" << desktopFileName;
         return;
     }
 
-    auto *job = new KIO::ApplicationLauncherJob(service);
+    auto job = new KIO::ApplicationLauncherJob(service);
     connect(job, &KJob::finished, this, [this, service](KJob *job) {
         if (job->error()) {
             Q_EMIT backend()->passiveMessage(i18n("Failed to start '%1': %2", service->name(), job->errorString()));
@@ -504,7 +534,14 @@ void FlatpakResource::invokeApplication() const
 
 void FlatpakResource::fetchChangelog()
 {
-    Q_EMIT changelogFetched(AppStreamUtils::changelogToHtml(m_appdata));
+    if (m_permissions.isEmpty()) {
+        loadPermissions();
+    }
+    QString changelog = AppStreamUtils::changelogToHtml(m_appdata);
+    if (!m_newPermissions.isEmpty()) {
+        changelog += "<h3>New Permissions</h3>\n<ul>"_L1 + m_newPermissions + "</ul>"_L1;
+    }
+    Q_EMIT changelogFetched(changelog);
 }
 
 void FlatpakResource::fetchScreenshots()
@@ -527,11 +564,6 @@ void FlatpakResource::setBundledIcon(const QPixmap &pixmap)
     m_bundledIcon = pixmap;
 }
 
-void FlatpakResource::setCommit(const QString &commit)
-{
-    m_commit = commit;
-}
-
 void FlatpakResource::setDownloadSize(quint64 size)
 {
     m_downloadSize = size;
@@ -602,7 +634,7 @@ void FlatpakResource::setState(AbstractResource::State state, bool shouldEmit)
     if (m_state != state) {
         m_state = state;
 
-        if (shouldEmit && qobject_cast<FlatpakBackend *>(backend())->isTracked(this)) {
+        if (shouldEmit && backend()->isTracked(this)) {
             Q_EMIT stateChanged();
         }
     }
@@ -625,23 +657,18 @@ QString FlatpakResource::installationPath(FlatpakInstallation *flatpakInstallati
     return QString::fromUtf8(path_str);
 }
 
-QString FlatpakResource::installPath() const
-{
-    return installationPath() + QStringLiteral("/app/%1/%2/%3/active").arg(flatpakName(), arch(), branch());
-}
-
 QUrl FlatpakResource::url() const
 {
     if (!m_resourceFile.isEmpty()) {
         return m_resourceFile;
     }
 
-    QUrl ret(QStringLiteral("appstream://") + appstreamId());
+    QUrl ret(QStringLiteral("appstream:") + appstreamId());
     const AppStream::Provided::Kind AppStream_Provided_KindId = (AppStream::Provided::Kind)12; // Should be AppStream::Provided::KindId when released
     const auto provided = m_appdata.provided(AppStream_Provided_KindId).items();
     if (!provided.isEmpty()) {
         QUrlQuery qq;
-        qq.addQueryItem("alt", provided.join(QLatin1Char(',')));
+        qq.addQueryItem(u"alt"_s, provided.join(QLatin1Char(',')));
         ret.setQuery(qq);
     }
     return ret;
@@ -649,9 +676,11 @@ QUrl FlatpakResource::url() const
 
 QDate FlatpakResource::releaseDate() const
 {
-    if (!m_appdata.releases().isEmpty()) {
-        auto release = m_appdata.releases().constFirst();
-        return release.timestamp().date();
+    if (const auto releases = m_appdata.releasesPlain(); !releases.isEmpty()) {
+        auto release = releases.indexSafe(0);
+        if (release) {
+            return release->timestamp().date();
+        }
     }
 
     return {};
@@ -659,21 +688,22 @@ QDate FlatpakResource::releaseDate() const
 
 QString FlatpakResource::sourceIcon() const
 {
-    const auto sourceItem = qobject_cast<FlatpakBackend *>(backend())->sources()->sourceById(origin());
+    const auto sourceItem = backend()->sources()->sourceById(origin());
     if (!sourceItem) {
-        qWarning() << "Could not find source " << origin();
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Could not find source " << origin();
         return QStringLiteral("flatpak-discover");
     }
 
     const auto iconUrl = sourceItem->data(FlatpakSourcesBackend::IconUrlRole).toString();
-    if (iconUrl.isEmpty())
+    if (iconUrl.isEmpty()) {
         return QStringLiteral("flatpak-discover");
+    }
     return iconUrl;
 }
 
 QString FlatpakResource::author() const
 {
-    QString name = m_appdata.developerName();
+    QString name = m_appdata.developer().name();
 
     if (name.isEmpty()) {
         name = m_appdata.projectGroup();
@@ -707,12 +737,12 @@ QString FlatpakResource::versionString()
         return {};
     }
     if (isInstalled()) {
-        auto ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
+        auto ref = backend()->getInstalledRefForApp(this);
         if (ref) {
-            version = flatpak_installed_ref_get_appdata_version(ref);
+            version = QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref));
         }
-    } else if (!m_appdata.releases().isEmpty()) {
-        auto release = m_appdata.releases().constFirst();
+    } else if (!m_appdata.releasesPlain().isEmpty()) {
+        const auto release = m_appdata.releasesPlain().indexSafe(0).value();
         version = release.version();
     } else {
         version = m_id.branch;
@@ -737,29 +767,65 @@ QString translateSymbolicName(const QStringView &name)
 
 QString FlatpakResource::eolReason()
 {
-    if (!m_eolReason.has_value()) {
-        auto futureWatcher = new QFutureWatcher<FlatpakRemoteRef *>(this);
-        connect(futureWatcher, &QFutureWatcher<FlatpakRemoteRef *>::finished, this, [this, futureWatcher]() {
-            g_autoptr(FlatpakRemoteRef) rref = futureWatcher->result();
-            if (rref) {
-                m_eolReason = QString::fromUtf8(flatpak_remote_ref_get_eol(rref));
-                Q_EMIT eolReasonChanged();
-            }
-            futureWatcher->deleteLater();
-        });
+    if (!m_eolReason.has_value() && !m_eolReasonTask.has_value()) {
+        QPointer<FlatpakResource> self = this;
 
-        futureWatcher->setFuture(QtConcurrent::run(qobject_cast<FlatpakBackend *>(backend())->threadPool(),
-                                                   &FlatpakRunnables::findRemoteRef,
-                                                   this,
-                                                   qobject_cast<FlatpakBackend *>(backend())->cancellable()));
-        return {};
+        m_eolReasonTask = [](FlatpakResource *self) -> QCoro::Task<std::optional<QString>> {
+            QPointer<FlatpakResource> guard = self;
+            auto backend = qobject_cast<FlatpakBackend *>(self->backend());
+            auto cancellable = backend->cancellable();
+
+            g_autoptr(FlatpakRemoteRef) rref = co_await QtConcurrent::run(backend->threadPool(), &FlatpakRunnables::findRemoteRef, self, cancellable);
+
+            if (guard && !g_cancellable_is_cancelled(cancellable) && rref) {
+                co_return QString::fromUtf8(flatpak_remote_ref_get_eol(rref));
+            }
+
+            co_return std::nullopt;
+        }(this)
+                                                           .then([guard = QPointer(this)](std::optional<QString> reason) {
+                                                               if (!guard) {
+                                                                   return;
+                                                               }
+                                                               guard->m_eolReasonTask.reset();
+                                                               if (reason.has_value()) {
+                                                                   guard->setEolReason(reason.value());
+                                                               }
+                                                           });
     }
     return m_eolReason.value_or(QString());
 }
 
+void FlatpakResource::setEolReason(const QString &reason)
+{
+    if (m_eolReason != reason) {
+        m_eolReason = reason;
+        Q_EMIT eolReasonChanged();
+    }
+}
+
+QString createHtmlList(const QStringList &itemList)
+{
+    QString str = QStringLiteral("<ul>");
+    for (const QString &itemText : std::as_const(itemList)) {
+        str += QStringLiteral("<li>%1</li>").arg(itemText.toHtmlEscaped());
+    }
+    str += QStringLiteral("</ul>");
+    return str;
+}
+
+bool FlatpakResource::updateNeedsAttention()
+{
+    if (m_permissions.isEmpty()) {
+        loadPermissions();
+    }
+    return !m_newPermissions.isEmpty();
+}
+
 void FlatpakResource::loadPermissions()
 {
-    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, NULL);
+    auto b = qobject_cast<FlatpakBackend *>(backend());
+    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, b->cancellable());
 
     QTemporaryFile f;
     if (!f.open()) {
@@ -767,82 +833,100 @@ void FlatpakResource::loadPermissions()
     }
     f.write(metaDataBytes);
     f.close();
+    m_permissions = readPermissions(f.fileName());
 
-    KDesktopFile parser(f.fileName());
+    auto currentPermissions = readPermissions(installPath() + "/metadata"_L1);
+    for (const auto &permission : std::as_const(m_permissions)) {
+        if (!currentPermissions.contains(permission)) {
+            m_newPermissions += "<li>"_L1 + i18n("%1: %2", permission.brief(), permission.description()) + "</li>\n"_L1;
+        }
+    }
+}
+
+QVector<FlatpakPermission> FlatpakResource::readPermissions(const QString &file) const
+{
+    QVector<FlatpakPermission> permissions;
+    KDesktopFile parser(file);
 
     QString brief, description;
 
     bool fullSessionBusAccess = false;
     bool fullSystemBusAccess = false;
 
-    const KConfigGroup contextGroup = parser.group("Context");
+    const KConfigGroup contextGroup = parser.group(u"Context"_s);
     const QString shared = contextGroup.readEntry("shared", QString());
-    if (shared.contains("network")) {
+    if (shared.contains("network"_L1)) {
         brief = i18n("Network Access");
         description = i18n("Can access the internet");
-        m_permissions.append(FlatpakPermission(brief, description, "network-wireless"));
+        permissions.append(FlatpakPermission(brief, description, u"network-wireless-symbolic"_s));
     }
 
     const QString sockets = contextGroup.readEntry("sockets", QString());
-    if (sockets.contains("session-bus")) {
+    const QString filesystems = contextGroup.readEntry("filesystems", QString());
+    const auto dirs = QStringView(filesystems).split(';'_L1, Qt::SkipEmptyParts);
+
+    if (sockets.contains("pulseaudio"_L1) || filesystems.contains("xdg-run/pipewire-0"_L1)) {
+        brief = i18n("Sound system access");
+        description = i18n("Can play audio");
+        permissions.append(FlatpakPermission(brief, description, u"audio-speakers-symbolic"_s));
+    }
+    if (sockets.contains("session-bus"_L1)) {
         brief = i18n("Session Bus Access");
-        description = i18n("Access is granted to the entire Session Bus");
-        m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+        description = i18n("Can communicate with all other applications and processes run in this user account");
+        permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSessionBusAccess = true;
     }
-    if (sockets.contains("system-bus")) {
+    if (sockets.contains("system-bus"_L1)) {
         brief = i18n("System Bus Access");
-        description = i18n("Access is granted to the entire System Bus");
-        m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+        description = i18n("Can communicate with all other applications and processes on the system");
+        permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSystemBusAccess = true;
     }
-    if (sockets.contains("ssh-auth")) {
+    if (sockets.contains("ssh-auth"_L1)) {
         brief = i18n("Remote Login Access");
         description = i18n("Can initiate remote login requests using the SSH protocol");
-        m_permissions.append(FlatpakPermission(brief, description, "x-shape-connection"));
+        permissions.append(FlatpakPermission(brief, description, u"x-shape-connection-symbolic"_s));
     }
-    if (sockets.contains("pcsc")) {
+    if (sockets.contains("pcsc"_L1)) {
         brief = i18n("Smart Card Access");
         description = i18n("Can integrate and communicate with smart cards");
-        m_permissions.append(FlatpakPermission(brief, description, "network-card"));
+        permissions.append(FlatpakPermission(brief, description, u"auth-sim-symbolic"_s));
     }
-    if (sockets.contains("cups")) {
+    if (sockets.contains("cups"_L1)) {
         brief = i18n("Printer Access");
         description = i18n("Can integrate and communicate with printers");
-        m_permissions.append(FlatpakPermission(brief, description, "document-print"));
+        permissions.append(FlatpakPermission(brief, description, u"printer-symbolic"_s));
     }
-    if (sockets.contains("gpg-agent")) {
+    if (sockets.contains("gpg-agent"_L1)) {
         brief = i18n("GPG Agent");
         description = i18n("Allows access to the GPG cryptography service, generally used for signing and reading signed documents");
-        m_permissions.append(FlatpakPermission(brief, description, "gpg"));
+        permissions.append(FlatpakPermission(brief, description, u"document-edit-sign-encrypt"_s));
     }
 
     const QString features = contextGroup.readEntry("features", QString());
-    if (features.contains("bluetooth")) {
+    if (features.contains("bluetooth"_L1)) {
         brief = i18n("Bluetooth Access");
         description = i18n("Can integrate and communicate with Bluetooth devices");
-        m_permissions.append(FlatpakPermission(brief, description, "network-bluetooth"));
+        permissions.append(FlatpakPermission(brief, description, u"network-bluetooth-symbolic"_s));
     }
-    if (features.contains("devel")) {
+    if (features.contains("devel"_L1)) {
         brief = i18n("Low-Level System Access");
         description = i18n("Can make low-level system calls (e.g. ptrace)");
-        m_permissions.append(FlatpakPermission(brief, description, "project-development"));
+        permissions.append(FlatpakPermission(brief, description, u"run-build-symbolic"_s));
     }
 
     const QString devices = contextGroup.readEntry("devices", QString());
-    if (devices.contains("all")) {
+    if (devices.contains("all"_L1)) {
         brief = i18n("Device Access");
         description = i18n("Can communicate with and control built-in or connected hardware devices");
-        m_permissions.append(FlatpakPermission(brief, description, "preferences-devices-tree"));
+        permissions.append(FlatpakPermission(brief, description, u"device-notifier-symbolic"_s));
     }
-    if (devices.contains("kvm")) {
+    if (devices.contains("kvm"_L1)) {
         brief = i18n("Kernel-based Virtual Machine Access");
         description = i18n("Allows running other operating systems as guests in virtual machines");
-        m_permissions.append(FlatpakPermission(brief, description, "virtualbox"));
+        permissions.append(FlatpakPermission(brief, description, u"virt-manager-symbolic"_s));
     }
 
-    const QString filesystems = contextGroup.readEntry("filesystems", QString());
-    const auto dirs = QStringView(filesystems).split(';', Qt::SkipEmptyParts);
     QStringList homeList, systemList;
     bool home_ro = false, home_rw = false, home_cr = false, homeAccess = false;
     bool system_ro = false, system_rw = false, system_cr = false, systemAccess = false;
@@ -855,11 +939,16 @@ void FlatpakResource::loadPermissions()
             continue;
         }
 
-        int separator = dir.lastIndexOf(':');
+        if (dir.startsWith("xdg-run/pipewire-0"_L1)) {
+            // This is already handled as sound system above
+            continue;
+        }
+
+        int separator = dir.lastIndexOf(':'_L1);
         const QStringView postfix = separator > 0 ? dir.mid(separator) : QStringView();
         const QStringView symbolicName = dir.left(separator);
         const QString id = translateSymbolicName(symbolicName);
-        if ((dir.contains(QLatin1String("home")) || dir.contains(QChar('~')))) {
+        if ((dir.contains(QLatin1String("home")) || dir.contains(QLatin1Char('~')))) {
             if (postfix == QLatin1String(":ro")) {
                 homeList << i18n("%1 (read-only)", id);
                 home_ro = true;
@@ -892,7 +981,7 @@ void FlatpakResource::loadPermissions()
         }
     }
 
-    QString appendText = "\n- " + homeList.join("\n- ");
+    QString appendText = createHtmlList(homeList);
     if (homeAccess) {
         brief = i18n("Home Folder Access");
         if (home_rw && home_ro && home_cr) {
@@ -905,9 +994,9 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access files in the following locations in your home folder without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, "inode-directory"));
+        permissions.append(FlatpakPermission(brief, description, u"user-home-symbolic"_s));
     }
-    appendText = "\n- " + systemList.join("\n- ");
+    appendText = createHtmlList(systemList);
     if (systemAccess) {
         brief = i18n("System Folder Access");
         if (system_rw && system_ro && system_cr) {
@@ -919,35 +1008,37 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access system files in the following locations without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, "inode-directory"));
+        permissions.append(FlatpakPermission(brief, description, u"drive-harddisk-root-symbolic"_s));
     }
 
     if (!fullSessionBusAccess) {
-        const KConfigGroup sessionBusGroup = parser.group("Session Bus Policy");
+        const KConfigGroup sessionBusGroup = parser.group(u"Session Bus Policy"_s);
         if (sessionBusGroup.exists()) {
             const QStringList busList = sessionBusGroup.keyList();
             brief = i18n("Session Bus Access");
-            description = i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
-                           "\n- " + busList.join("\n- "));
-            m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+            description =
+                i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
+                     createHtmlList(busList));
+            permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
 
     if (!fullSystemBusAccess) {
-        const KConfigGroup systemBusGroup = parser.group("System Bus Policy");
+        const KConfigGroup systemBusGroup = parser.group(u"System Bus Policy"_s);
         if (systemBusGroup.exists()) {
             const QStringList busList = systemBusGroup.keyList();
             brief = i18n("System Bus Access");
             description =
-                i18n("Can communicate with all applications and system services using the following communication protocols: %1", "\n- " + busList.join("\n- "));
-            m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+                i18n("Can communicate with all applications and system services using the following communication protocols: %1", createHtmlList(busList));
+            permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
+    return permissions;
 }
 
 QString FlatpakResource::dataLocation() const
 {
-    auto id = m_appdata.bundle(AppStream::Bundle::KindFlatpak).id().section('/', 0, 1);
+    auto id = m_appdata.bundle(AppStream::Bundle::KindFlatpak).id().section('/'_L1, 0, 1);
     if (id.isEmpty()) {
         return {};
     }
@@ -963,18 +1054,24 @@ void FlatpakResource::clearUserData()
 {
     const auto location = dataLocation();
     if (location.isEmpty()) {
-        qWarning() << "Failed find location for" << name();
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed find location for" << name();
         return;
     }
 
     if (!QDir(location).removeRecursively()) {
-        qWarning() << "Failed to remove location" << location;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to remove location" << location;
     }
     Q_EMIT hasDataChanged();
 }
 
 int FlatpakResource::versionCompare(FlatpakResource *resource) const
 {
+    // This is getting called from QML, we better sanitize the input
+    if (!resource) {
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Cannot compare with a null resource";
+        return -1;
+    }
+
     const QString other = resource->availableVersion();
     return AppStream::Utils::vercmpSimple(availableVersion(), other);
 }
@@ -984,17 +1081,52 @@ QString FlatpakResource::contentRatingDescription() const
     return AppStreamUtils::contentRatingDescription(m_appdata);
 }
 
-QString FlatpakResource::contentRatingText() const
-{
-    return AppStreamUtils::contentRatingText(m_appdata);
-}
-
-AbstractResource::ContentIntensity FlatpakResource::contentRatingIntensity() const
-{
-    return AppStreamUtils::contentRatingIntensity(m_appdata);
-}
-
 uint FlatpakResource::contentRatingMinimumAge() const
 {
     return AppStreamUtils::contentRatingMinimumAge(m_appdata);
 }
+
+QStringList FlatpakResource::topObjects() const
+{
+    return s_topObjects;
+}
+
+QStringList FlatpakResource::bottomObjects() const
+{
+    // Don't need permissions for non-apps; it doesn't make any sense
+    if (isDesktopApp()) {
+        return s_bottomObjects;
+    }
+    return {};
+}
+
+void FlatpakResource::addRefToUpdate(const QByteArray &toUpdate)
+{
+    m_toUpdate += toUpdate;
+}
+
+void FlatpakResource::clearToUpdate()
+{
+    m_toUpdate.clear();
+}
+
+bool FlatpakResource::hasResolvedIcon() const
+{
+    return m_icon.has_value();
+}
+
+QString FlatpakResource::verifiedMessage() const
+{
+    const bool flathubVerified = m_appdata.customValue(QStringLiteral("flathub::verification::verified")) == QLatin1String("true");
+    if (flathubVerified) {
+        bool ok;
+        const qint64 time = m_appdata.customValue(QStringLiteral("flathub::verification::timestamp")).toUInt(&ok);
+        return i18n("App Verified on %1 as shipped by its author %2",
+                    KFormat().formatRelativeDateTime(QDateTime::fromSecsSinceEpoch(time), QLocale::ShortFormat),
+                    author());
+    }
+    return {};
+}
+
+#include "FlatpakResource.moc"
+#include "moc_FlatpakResource.cpp"
