@@ -12,17 +12,14 @@
 #include <QStandardPaths>
 #include <QTimer>
 
-// Attica includes
-#include <Attica/Content>
-#include <Attica/ProviderManager>
-
 // KDE includes
 #include <KConfig>
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <KNSCore/Engine>
+#include <KNSCore/Provider>
+#include <KNSCore/Question>
 #include <KNSCore/QuestionManager>
-#include <knewstuffcore_version.h>
+#include <KNSCore/ResultsStream>
 
 // DiscoverCommon includes
 #include "Category/Category.h"
@@ -37,6 +34,8 @@
 #include "utils.h"
 #include <resources/StandardBackendUpdater.h>
 
+using namespace Qt::StringLiterals;
+
 static const int ENGINE_PAGE_SIZE = 100;
 
 class KNSBackendFactory : public AbstractResourcesBackendFactory
@@ -69,7 +68,7 @@ public:
                     return;
                 }
             }
-            qWarning() << "Question for unknown resource" << question->question() << question->questionType();
+            qWarning() << "Question for unknown transaction" << question->question() << question->questionType();
             question->setResponse(KNSCore::Question::InvalidResponse);
         });
     }
@@ -77,7 +76,7 @@ public:
     QVector<AbstractResourcesBackend *> newInstance(QObject *parent, const QString & /*name*/) const override
     {
         QVector<AbstractResourcesBackend *> ret;
-        const QStringList availableConfigFiles = KNSCore::Engine::availableConfigFiles();
+        const QStringList availableConfigFiles = KNSCore::EngineBase::availableConfigFiles();
         for (const QString &configFile : availableConfigFiles) {
             auto bk = new KNSBackend(parent, QStringLiteral("plasma"), configFile);
             if (bk->isValid())
@@ -89,9 +88,58 @@ public:
     }
 };
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-Q_DECLARE_METATYPE(KNSCore::EntryInternal)
-#endif
+class KNSResultsStream : public ResultsStream
+{
+    Q_OBJECT
+public:
+    KNSResultsStream(KNSBackend *backend, const QString &objectName)
+        : ResultsStream(objectName)
+        , m_backend(backend)
+    {
+        // Abort streams when the backend gets invalidated
+        connect(m_backend, &KNSBackend::invalidated, this, &KNSResultsStream::finish);
+
+        if (!m_backend->isValid()) {
+            qWarning() << "Erroneously starting a kns stream with an invalid backend" << m_backend->name();
+        }
+    }
+
+    void setRequest(const KNSCore::SearchRequest &request)
+    {
+        Q_ASSERT(!m_started);
+        m_started = true;
+        KNSCore::ResultsStream *job = m_backend->engine()->search(request);
+        connect(job, &KNSCore::ResultsStream::entriesFound, this, &KNSResultsStream::addEntries);
+        connect(job, &KNSCore::ResultsStream::finished, this, &KNSResultsStream::finish);
+        connect(this, &ResultsStream::fetchMore, job, &KNSCore::ResultsStream::fetchMore);
+        job->fetch();
+    }
+
+    void addEntries(const KNSCore::Entry::List &entries)
+    {
+        // Should probably address that KNSCore::ResultsStream would return the Entry several times...
+        const auto selectedEntries = kFilter<KNSCore::Entry::List>(entries, [this](const auto &entry) {
+            return !m_sent.contains(entry.uniqueId());
+        });
+        const auto res = kTransform<QList<StreamResult>>(selectedEntries, [this](const auto &entry) {
+            return StreamResult{m_backend->resourceForEntry(entry), 0};
+        });
+        for (const auto &entry : selectedEntries) {
+            m_sent.insert(entry.uniqueId());
+        }
+        Q_EMIT resourcesFound(res);
+    }
+
+    bool hasStarted() const
+    {
+        return m_started;
+    }
+
+private:
+    QSet<QString> m_sent;
+    KNSBackend *const m_backend;
+    bool m_started = false;
+};
 
 KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &knsrc)
     : AbstractResourcesBackend(parent)
@@ -107,14 +155,15 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     setObjectName(knsrc);
 
     const KConfig conf(m_name, KConfig::SimpleConfig);
-    if (!conf.hasGroup("KNewStuff3")) {
-        markInvalid(QStringLiteral("Config group not found! Check your KNS3 installation."));
+    const bool hasVersionlessGrp = conf.hasGroup(u"KNewStuff"_s);
+    if (!conf.hasGroup(u"KNewStuff3"_s) && !hasVersionlessGrp) {
+        markInvalid(QStringLiteral("Config group not found! Check your KNSCore installation."));
         return;
     }
 
     m_categories = QStringList{fileName};
 
-    const KConfigGroup group = conf.group("KNewStuff3");
+    const KConfigGroup group = hasVersionlessGrp ? conf.group(u"KNewStuff"_s) : conf.group(u"KNewStuff3"_s);
     m_extends = group.readEntry("Extends", QStringList());
 
     setFetching(true);
@@ -127,7 +176,6 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     QTimer::singleShot(60000, this, [this]() {
         if (!m_initialized) {
             markInvalid(i18n("Backend %1 took too long to initialize", m_displayName));
-            setResponsePending(false);
         }
     });
 
@@ -153,9 +201,9 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     }
 
     QVector<Category *> topCategories{categories};
-    for (const auto &cat : qAsConst(categories)) {
+    for (const auto &cat : std::as_const(categories)) {
         const QString catName = cat->name().append(QLatin1Char('/'));
-        for (const auto &potentialSubCat : qAsConst(categories)) {
+        for (const auto &potentialSubCat : std::as_const(categories)) {
             if (potentialSubCat->name().startsWith(catName)) {
                 cat->addSubcategory(potentialSubCat);
                 topCategories.removeOne(potentialSubCat);
@@ -163,27 +211,22 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         }
     }
 
-    m_engine = new KNSCore::Engine(this);
-    connect(m_engine, &KNSCore::Engine::signalErrorCode, this, &KNSBackend::slotErrorCode);
-    connect(m_engine, &KNSCore::Engine::signalEntryEvent, this, &KNSBackend::slotEntryEvent);
-    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
-    connect(m_engine, &KNSCore::Engine::signalUpdateableEntriesLoaded, this, [this] {
-        if (m_responsePending) {
-            setResponsePending(false);
-        }
+    m_engine = new KNSCore::EngineBase(this);
+    connect(m_engine, &KNSCore::EngineBase::signalErrorCode, this, &KNSBackend::slotErrorCode);
+    connect(m_engine, &KNSCore::EngineBase::providerAdded, this, [this] {
+        setFetching(false);
     });
-    connect(m_engine, &KNSCore::Engine::signalCategoriesMetadataLoded, this, [categories](const QList<KNSCore::Provider::CategoryMetadata> &categoryMetadatas) {
-        for (const KNSCore::Provider::CategoryMetadata &category : categoryMetadatas) {
-            for (Category *cat : qAsConst(categories)) {
-                if (cat->matchesCategoryName(category.name)) {
-                    cat->setName(category.displayName);
+
+    connect(m_engine, &KNSCore::EngineBase::signalCategoriesMetadataLoaded, this, [categories](const QList<KNSCore::CategoryMetadata> &categoryMetadatas) {
+        for (const KNSCore::CategoryMetadata &category : categoryMetadatas) {
+            for (Category *cat : std::as_const(categories)) {
+                if (cat->matchesCategoryName(category.name())) {
+                    cat->setName(category.displayName());
                     break;
                 }
             }
         }
     });
-    m_engine->setPageSize(ENGINE_PAGE_SIZE);
     m_engine->init(m_name);
 
     if (m_hasApplications) {
@@ -254,95 +297,14 @@ void KNSBackend::markInvalid(const QString &message)
     m_isValid = false;
     setFetching(false);
     Q_EMIT initialized();
-}
-
-void KNSBackend::setResponsePending(bool pending)
-{
-    Q_ASSERT(m_responsePending != pending);
-    m_responsePending = pending;
-    if (pending) {
-        Q_EMIT startingSearch();
-    } else {
-        Q_EMIT availableForQueries();
-        setFetching(false);
-        m_onePage = false;
-    }
-}
-
-void KNSBackend::fetchInstalled()
-{
-    auto search = new OneTimeAction(
-        666,
-        [this]() {
-            // First we ensure we've got data loaded on what we've got installed already
-            if (m_responsePending) {
-                // Slot already taken, will need to wait again
-                return false;
-            }
-            m_onePage = true;
-            setResponsePending(true);
-            m_engine->checkForInstalled();
-            // And then we check for updates - we could do only one, if all we cared about was updates,
-            // but to have both a useful initial list, /and/ information on updates, we want to get both.
-            // The reason we are not doing a checkUpdates() overload for this is that the caching for this
-            // information is done by KNSEngine, and we want to actually load it every time we initialize.
-            auto updateChecker = new OneTimeAction(
-                666,
-                [this] {
-                    // No need to check for updates if there's no resources
-                    if (m_resourcesByName.isEmpty()) {
-                        return true;
-                    }
-
-                    if (m_responsePending) {
-                        // Slot already taken, will need to wait again
-                        return false;
-                    }
-
-                    m_onePage = true;
-                    setResponsePending(true);
-                    m_engine->checkForUpdates();
-                    return true;
-                },
-                this);
-            connect(this, &KNSBackend::availableForQueries, updateChecker, &OneTimeAction::trigger, Qt::QueuedConnection);
-            return true;
-        },
-        this);
-
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, search, &OneTimeAction::trigger, Qt::QueuedConnection);
-    } else {
-        search->trigger();
-    }
+    Q_EMIT invalidated();
 }
 
 void KNSBackend::checkForUpdates()
 {
-    // Since we load the updates during initialization already, don't overburden
-    // the machine with multiple of these, because that would just be silly.
-    if (m_initialized) {
-        auto updateChecker = new OneTimeAction(
-            666,
-            [this] {
-                if (m_responsePending) {
-                    // Slot already taken, will need to wait again
-                    return false;
-                }
-
-                m_onePage = true;
-                setResponsePending(true);
-                m_engine->checkForUpdates();
-                return true;
-            },
-            this);
-
-        if (m_responsePending) {
-            connect(this, &KNSBackend::availableForQueries, updateChecker, &OneTimeAction::trigger, Qt::QueuedConnection);
-        } else {
-            updateChecker->trigger();
-        }
-    }
+    AbstractResourcesBackend::Filters filter;
+    filter.state = AbstractResource::Upgradeable;
+    search(filter);
 }
 
 void KNSBackend::setFetching(bool f)
@@ -362,17 +324,20 @@ bool KNSBackend::isValid() const
     return m_isValid;
 }
 
-KNSResource *KNSBackend::resourceForEntry(const KNSCore::EntryInternal &entry)
+KNSResource *KNSBackend::resourceForEntry(const KNSCore::Entry &entry)
 {
     KNSResource *r = static_cast<KNSResource *>(m_resourcesByName.value(entry.uniqueId()));
     if (!r) {
-        QStringList categories{name(), m_rootCategories.first()->name()};
-        const auto cats = m_engine->categoriesMetadata();
-        const int catIndex = kIndexOf(cats, [&entry](const KNSCore::Provider::CategoryMetadata &cat) {
-            return entry.category() == cat.id;
+        QStringList categories{name()};
+        if (!m_rootCategories.isEmpty()) {
+            categories << m_rootCategories.first()->name();
+        }
+        const auto cats = m_engine->categoriesMetadata2();
+        const int catIndex = kIndexOf(cats, [&entry](const KNSCore::CategoryMetadata &cat) {
+            return entry.category() == cat.id();
         });
         if (catIndex > -1) {
-            categories << cats.at(catIndex).name;
+            categories << cats.at(catIndex).name();
         }
         if (m_hasApplications) {
             categories << QLatin1String("Application");
@@ -385,46 +350,12 @@ KNSResource *KNSBackend::resourceForEntry(const KNSCore::EntryInternal &entry)
     return r;
 }
 
-void KNSBackend::receivedEntries(const KNSCore::EntryInternal::List &entries)
-{
-    if (!m_isValid)
-        return;
-
-    const auto filtered = kFilter<KNSCore::EntryInternal::List>(entries, [](const KNSCore::EntryInternal &entry) {
-        return entry.isValid();
-    });
-    const auto resources = kTransform<QVector<AbstractResource *>>(filtered, [this](const KNSCore::EntryInternal &entry) {
-        return resourceForEntry(entry);
-    });
-
-    if (!resources.isEmpty()) {
-        Q_EMIT receivedResources(resources);
-    }
-
-    setResponsePending(false);
-    if (m_onePage || resources.count() < ENGINE_PAGE_SIZE) {
-        Q_EMIT searchFinished();
-    }
-}
-
-void KNSBackend::fetchMore()
-{
-    if (m_responsePending)
-        return;
-
-    // We _have_ to set this first. If we do not, we may run into a situation where the
-    // data request will conclude immediately, causing m_responsePending to remain true
-    // for perpetuity as the slots will be called before the function returns.
-    setResponsePending(true);
-    m_engine->requestMoreData();
-}
-
-void KNSBackend::statusChanged(const KNSCore::EntryInternal &entry)
+void KNSBackend::statusChanged(const KNSCore::Entry &entry)
 {
     resourceForEntry(entry);
 }
 
-void KNSBackend::slotErrorCode(const KNSCore::ErrorCode &errorCode, const QString &message, const QVariant &metadata)
+void KNSBackend::slotErrorCode(const KNSCore::ErrorCode::ErrorCode &errorCode, const QString &message, const QVariant &metadata)
 {
     QString error = message;
     qWarning() << "KNS error in" << m_displayName << ":" << errorCode << message << metadata;
@@ -473,7 +404,7 @@ void KNSBackend::slotErrorCode(const KNSCore::ErrorCode &errorCode, const QStrin
             // query to the user) but we can give them an idea of how to deal with the
             // situation some other way.
             // TODO: Once Discover has a way to forward queries to the user from transactions, this likely will no longer be needed
-            if (r->entry().status() == KNS3::Entry::Updateable) {
+            if (r->entry().status() == KNSCore::Entry::Updateable) {
                 error = i18n(
                     "Unable to complete the update of %1. You can try and perform this action through the Get Hot New Stuff dialog, which grants tighter "
                     "control. The reported error was:\n%2",
@@ -492,25 +423,22 @@ void KNSBackend::slotErrorCode(const KNSCore::ErrorCode &errorCode, const QStrin
         error = i18n("Unhandled error in %1 backend. Contact your distributor.", m_displayName);
         break;
     }
-    if (m_responsePending) {
-        setResponsePending(false);
-    }
     qWarning() << "kns error" << objectName() << error;
     if (!invalidFile)
         Q_EMIT passiveMessage(i18n("%1: %2", name(), error));
 }
 
-void KNSBackend::slotEntryEvent(const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event)
+void KNSBackend::slotEntryEvent(const KNSCore::Entry &entry, KNSCore::Entry::EntryEvent event)
 {
     switch (event) {
-    case KNSCore::EntryInternal::StatusChangedEvent:
+    case KNSCore::Entry::StatusChangedEvent:
         statusChanged(entry);
         break;
-    case KNSCore::EntryInternal::DetailsLoadedEvent:
+    case KNSCore::Entry::DetailsLoadedEvent:
         detailsLoaded(entry);
         break;
-    case KNSCore::EntryInternal::AdoptedEvent:
-    case KNSCore::EntryInternal::UnknownEvent:
+    case KNSCore::Entry::AdoptedEvent:
+    case KNSCore::Entry::UnknownEvent:
     default:
         break;
     }
@@ -548,6 +476,25 @@ static ResultsStream *voidStream()
     return new ResultsStream(QStringLiteral("KNS-void"), {});
 }
 
+template<typename T>
+void KNSBackend::deferredResultStream(KNSResultsStream *stream, T start)
+{
+    if (isFetching()) {
+        auto startOnce = [stream, start] {
+            if (stream->hasStarted()) {
+                return;
+            }
+            start();
+        };
+
+        // If it's not ready to take queries, wait a bit longer
+        connect(this, &KNSBackend::initialized, stream, startOnce, Qt::QueuedConnection);
+        connect(this, &KNSBackend::fetchingChanged, stream, startOnce, Qt::QueuedConnection);
+    } else {
+        QTimer::singleShot(0, stream, start);
+    }
+}
+
 ResultsStream *KNSBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
     if (!m_isValid || (!filter.resourceUrl.isEmpty() && filter.resourceUrl.scheme() != QLatin1String("kns")) || !filter.mimetype.isEmpty())
@@ -556,45 +503,28 @@ ResultsStream *KNSBackend::search(const AbstractResourcesBackend::Filters &filte
     if (filter.resourceUrl.scheme() == QLatin1String("kns")) {
         return findResourceByPackageName(filter.resourceUrl);
     } else if (filter.state >= AbstractResource::Installed) {
-        auto stream = new ResultsStream(QStringLiteral("KNS-installed"));
-
+        auto stream = new KNSResultsStream(this, "KNS-installed-"_L1 + name());
         const auto start = [this, stream, filter]() {
             if (m_isValid) {
-                auto filterFunction = [&filter](AbstractResource *r) {
-                    return r->state() >= filter.state
-                        && (r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive));
-                };
-                const auto ret = kFilter<QVector<AbstractResource *>>(m_resourcesByName, filterFunction);
-
-                if (!ret.isEmpty())
-                    Q_EMIT stream->resourcesFound(ret);
+                const auto knsFilter = filter.state == AbstractResource::Installed ? KNSCore::Filter::Installed : KNSCore::Filter::Updates;
+                stream->setRequest(KNSCore::SearchRequest(KNSCore::SortMode::Newest, knsFilter, {}, {}, -1, ENGINE_PAGE_SIZE));
             }
-            stream->finish();
         };
-        if (isFetching()) {
-            connect(this, &KNSBackend::initialized, stream, start);
-        } else {
-            QTimer::singleShot(0, stream, start);
-        }
-
+        deferredResultStream(stream, start);
         return stream;
     } else if ((!filter.category && !filter.search.isEmpty()) // Accept global searches
                                                               // If there /is/ a category, make sure we actually are one of those requested before searching
                || (filter.category && kContains(m_categories, [&filter](const QString &cat) {
                        return filter.category->matchesCategoryName(cat);
                    }))) {
-        auto r = new ResultsStream(QLatin1String("KNS-search-") + name());
-        searchStream(r, filter.search);
-        return r;
+        return searchStream(filter.search);
     }
     return voidStream();
 }
 
-void KNSBackend::searchStream(ResultsStream *stream, const QString &searchText)
+KNSResultsStream *KNSBackend::searchStream(const QString &searchText)
 {
-    Q_EMIT startingSearch();
-
-    stream->setProperty("alreadyStarted", false);
+    auto stream = new KNSResultsStream(this, QLatin1String("KNS-search-") + name());
     auto start = [this, stream, searchText]() {
         Q_ASSERT(!isFetching());
         if (!m_isValid) {
@@ -602,30 +532,11 @@ void KNSBackend::searchStream(ResultsStream *stream, const QString &searchText)
             stream->finish();
             return;
         }
-
-        if (m_responsePending || stream->property("alreadyStarted").toBool()) {
-            return;
-        }
-        stream->setProperty("alreadyStarted", true);
-        setResponsePending(true);
-
-        // No need to explicitly launch a search, setting the search term already does that for us
-        m_engine->setSearchTerm(searchText);
-        m_onePage = false;
-
-        connect(stream, &ResultsStream::fetchMore, this, &KNSBackend::fetchMore);
-        connect(this, &KNSBackend::receivedResources, stream, &ResultsStream::resourcesFound);
-        connect(this, &KNSBackend::searchFinished, stream, &ResultsStream::finish);
-        connect(this, &KNSBackend::startingSearch, stream, &ResultsStream::finish);
+        KNSCore::SearchRequest p(KNSCore::SortMode::Newest, KNSCore::Filter::None, searchText, {}, 0, ENGINE_PAGE_SIZE);
+        stream->setRequest(p);
     };
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
-    } else if (isFetching()) {
-        connect(this, &KNSBackend::initialized, stream, start, Qt::QueuedConnection);
-        connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
-    } else {
-        QTimer::singleShot(0, stream, start);
-    }
+    deferredResultStream(stream, start);
+    return stream;
 }
 
 ResultsStream *KNSBackend::findResourceByPackageName(const QUrl &search)
@@ -634,52 +545,18 @@ ResultsStream *KNSBackend::findResourceByPackageName(const QUrl &search)
         return voidStream();
 
     const auto pathParts = search.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
-    if (pathParts.size() != 2) {
+    if (pathParts.size() != 1) {
         Q_EMIT passiveMessage(i18n("Wrong KNewStuff URI: %1", search.toString()));
         return voidStream();
     }
-    const auto providerid = pathParts.at(0);
-    const auto entryid = pathParts.at(1);
+    const auto entryid = pathParts.at(0);
 
-    auto stream = new ResultsStream(QLatin1String("KNS-byname-") + entryid);
-    auto start = [this, entryid, stream, providerid]() {
-        if (m_responsePending) {
-            // Slot already taken, will need to wait again
-            return;
-        }
-        setResponsePending(true);
-        m_engine->fetchEntryById(entryid);
-        m_onePage = true;
-
-        connect(m_engine, &KNSCore::Engine::signalErrorCode, stream, &ResultsStream::finish);
-        connect(m_engine,
-                &KNSCore::Engine::signalEntryEvent,
-                stream,
-                [this, stream, entryid, providerid](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
-                    switch (event) {
-                    case KNSCore::EntryInternal::StatusChangedEvent:
-                        if (entry.uniqueId() == entryid && providerid == QUrl(entry.providerId()).host()) {
-                            Q_EMIT stream->resourcesFound({resourceForEntry(entry)});
-                        } else
-                            qWarning() << "found invalid" << entryid << entry.uniqueId() << providerid << QUrl(entry.providerId()).host();
-                        QTimer::singleShot(0, this, [this] {
-                            setResponsePending(false);
-                        });
-                        stream->finish();
-                        break;
-                    case KNSCore::EntryInternal::DetailsLoadedEvent:
-                    case KNSCore::EntryInternal::AdoptedEvent:
-                    case KNSCore::EntryInternal::UnknownEvent:
-                    default:
-                        break;
-                    }
-                });
+    auto stream = new KNSResultsStream(this, QLatin1String("KNS-byname-") + entryid);
+    auto start = [entryid, stream]() {
+        KNSCore::SearchRequest query(KNSCore::SortMode::Newest, KNSCore::Filter::ExactEntryId, entryid, {}, 0, ENGINE_PAGE_SIZE);
+        stream->setRequest(query);
     };
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, stream, start);
-    } else {
-        start();
-    }
+    deferredResultStream(stream, start);
     return stream;
 }
 
@@ -698,10 +575,11 @@ QString KNSBackend::displayName() const
     return QStringLiteral("KNewStuff");
 }
 
-void KNSBackend::detailsLoaded(const KNSCore::EntryInternal &entry)
+void KNSBackend::detailsLoaded(const KNSCore::Entry &entry)
 {
     auto res = resourceForEntry(entry);
     Q_EMIT res->longDescriptionChanged();
 }
 
 #include "KNSBackend.moc"
+#include "moc_KNSBackend.cpp"
