@@ -8,12 +8,8 @@
 
 #include "PackageKitResource.h"
 
-#ifdef DISCOVER_USE_STABLE_APPSTREAM
-#include <AppStreamQt5/pool.h>
-#else
 #include <AppStreamQt/pool.h>
-#endif
-
+#include <PackageKit/Offline>
 #include <PackageKit/Transaction>
 #include <QFile>
 #include <QPointer>
@@ -97,13 +93,10 @@ public:
 
     bool isPackageNameUpgradeable(const PackageKitResource *res) const;
     QSet<QString> upgradeablePackageId(const PackageKitResource *res) const;
-    QVector<AppPackageKitResource *> extendedBy(const QString &id) const;
+    QVector<AbstractResource *> extendedBy(const QString &id) const;
 
     PKResolveTransaction *resolvePackages(const QStringList &packageNames);
-    void fetchDetails(const QString &pkgid)
-    {
-        m_details.add(pkgid);
-    }
+    void fetchDetails(const QString &pkgid);
     void fetchDetails(const QSet<QString> &pkgid);
 
     void checkForUpdates() override;
@@ -115,18 +108,35 @@ public:
     }
     static QString locateService(const QString &filename);
 
-    QList<AppStream::Component> componentsById(const QString &id) const;
+    AppStream::ComponentBox componentsById(const QString &id) const;
     void fetchUpdates();
     int fetchingUpdatesProgress() const override;
+    uint fetchingUpdatesProgressWeight() const override;
 
     InlineMessage *explainDysfunction() const override;
 
     void addPackageArch(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary);
     void addPackageNotArch(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary);
+    void clear()
+    {
+        m_updatesPackageId.clear();
+    }
     Delay &updateDetails()
     {
         return m_updateDetails;
     }
+    template<typename T, typename W>
+    T resourcesByPackageNames(const W &names) const;
+
+    QStringList globalHints()
+    {
+        return m_globalHints;
+    }
+    void aboutTo(AboutToAction action) override;
+    bool needsRebootForPowerOffAction() const override
+    {
+        return true;
+    }
 
 public Q_SLOTS:
     void reloadPackageList();
@@ -138,6 +148,7 @@ private Q_SLOTS:
     void packageDetails(const PackageKit::Details &details);
     void addPackageToUpdate(PackageKit::Transaction::Info, const QString &pkgid, const QString &summary);
     void getUpdatesFinished(PackageKit::Transaction::Exit, uint);
+    void loadAllPackages();
 
 Q_SIGNALS:
     void loadedAppStream();
@@ -145,44 +156,47 @@ Q_SIGNALS:
 
 private:
     friend class PackageKitResource;
-    template<typename T, typename W>
-    T resourcesByPackageNames(const W &names) const;
 
     template<typename T, typename W>
     T resourcesByAppNames(const W &names) const;
 
     template<typename T>
-    T resourcesByComponents(const QList<AppStream::Component> &names) const;
+    T resourcesByComponents(const AppStream::ComponentBox &names) const;
 
-    void runWhenInitialized(const std::function<void()> &f, QObject *stream);
+    QVector<StreamResult> resultsByComponents(const AppStream::ComponentBox &names) const;
+
+    PKResultsStream *deferredResultStream(const QString &streamName, std::function<void(PKResultsStream *)> callback);
 
     void checkDaemonRunning();
     void acquireFetching(bool f);
     void includePackagesToAdd();
     void performDetailsFetch(const QSet<QString> &pkgids);
-    AppPackageKitResource *addComponent(const AppStream::Component &component);
+    AppPackageKitResource *addComponent(const AppStream::Component &component) const;
     void updateProxy();
+    void foundNewMajorVersion(const AppStream::Release &release);
 
     QScopedPointer<AppStream::Pool> m_appdata;
+    bool m_appdataLoaded = false;
     PackageKitUpdater *m_updater;
     QPointer<PackageKit::Transaction> m_refresher;
     int m_isFetching;
     QSet<QString> m_updatesPackageId;
     bool m_hasSecurityUpdates = false;
-    QSet<PackageKitResource *> m_packagesToAdd;
+    mutable QHash<PackageOrAppId, PackageKitResource *> m_packagesToAdd;
     QSet<PackageKitResource *> m_packagesToDelete;
     bool m_appstreamInitialized = false;
 
-    struct {
+    mutable struct {
         QHash<PackageOrAppId, AbstractResource *> packages;
         QHash<QString, QStringList> packageToApp;
-        QHash<QString, QVector<AppPackageKitResource *>> extendedBy;
     } m_packages;
 
     Delay m_details;
     Delay m_updateDetails;
     QSharedPointer<OdrsReviewsBackend> m_reviews;
-    QPointer<PackageKit::Transaction> m_getUpdatesTransaction;
     QThreadPool m_threadPool;
     QPointer<PKResolveTransaction> m_resolveTransaction;
+    QStringList m_globalHints;
+    bool m_allPackagesLoaded = false;
+    DiscoverAction *m_reportToDistroAction;
 };
