@@ -5,10 +5,15 @@
  */
 #include "PackageKitUpdater.h"
 #include "PackageKitMessages.h"
+#include <AppStreamQt/release.h>
+#include <appstream/AppStreamIntegration.h>
 
 #include <PackageKit/Daemon>
 #include <PackageKit/Offline>
 #include <QCryptographicHash>
+#include <QDBusConnection>
+#include <QDBusMessage>
+#include <QDBusReply>
 #include <QDebug>
 #include <QSet>
 
@@ -18,10 +23,14 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
-#include "libdiscover_backend_debug.h"
+#include <optional>
+
+#include "libdiscover_backend_packagekit_debug.h"
 #include "pk-offline-private.h"
 #include "utils.h"
 
+using namespace Qt::StringLiterals;
+
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
 {
     const auto was = percentage;
@@ -35,12 +44,12 @@ int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage
         };
         const auto idx = statuses.value(status, -1);
         if (idx < 0) {
-            qCDebug(LIBDISCOVER_BACKEND_LOG) << "Status not present" << status << "among" << statuses.keys() << percentage;
+            qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "Status not present" << status << "among" << statuses.keys() << percentage;
             return -1;
         }
         percentage = (idx * 100 + percentage) / 2 /*the maximum in statuses*/;
     }
-    qCDebug(LIBDISCOVER_BACKEND_LOG) << "reporting progress with status:" << status << percentage << was;
+    qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "reporting progress with status:" << status << percentage << was;
     return percentage;
 }
 
@@ -67,8 +76,8 @@ public:
         , m_backend(backend)
         , m_updateSizeTimer(new QTimer(this))
     {
-        connect(m_backend, &AbstractResourcesBackend::resourceRemoved, this, [this](AbstractResource *res) {
-            m_resources.remove(res);
+        connect(m_backend, &AbstractResourcesBackend::resourceRemoved, this, [this](AbstractResource *resource) {
+            m_resources.remove(resource);
         });
 
         m_updateSizeTimer->setInterval(100);
@@ -82,6 +91,9 @@ public:
     }
     QString name() const override
     {
+        if (isDistroUpgrade()) {
+            return i18nc("distro upgrade: name version", "%1 %2", AppStreamIntegration::global()->osRelease()->name(), m_distroUpgrade->version());
+        }
         return i18n("System upgrade");
     }
     QString comment() override
@@ -103,13 +115,13 @@ public:
     {
         return Upgradeable;
     }
-    QStringList categories() override
+    bool hasCategory(const QString & /*category*/) const override
     {
         return {};
     }
     AbstractResource::Type type() const override
     {
-        return Technical;
+        return System;
     }
     bool isRemovable() const override
     {
@@ -120,12 +132,12 @@ public:
     {
         QVector<PackageKitResource *> ret;
         QSet<QString> donePkgs;
-        for (auto res : qAsConst(m_resources)) {
-            PackageKitResource *app = qobject_cast<PackageKitResource *>(res);
-            QString pkgname = app->packageName();
-            if (!donePkgs.contains(pkgname)) {
-                donePkgs.insert(pkgname);
-                ret += app;
+        for (auto resource : std::as_const(m_resources)) {
+            const auto pkResource = qobject_cast<PackageKitResource *>(resource);
+            const auto pkgName = pkResource->packageName();
+            if (!donePkgs.contains(pkgName)) {
+                donePkgs.insert(pkgName);
+                ret += pkResource;
             }
         }
         return ret;
@@ -134,16 +146,20 @@ public:
     quint64 size() override
     {
         quint64 ret = 0;
+        if (isDistroUpgrade()) {
+            return ret;
+        }
+
         const auto resources = withoutDuplicates();
-        for (auto res : resources) {
-            ret += res->size();
+        for (auto pkResource : resources) {
+            ret += pkResource->size();
         }
         return ret;
     }
     QJsonArray licenses() override
     {
         QJsonArray ret;
-        for (auto res : qAsConst(m_resources)) {
+        for (auto res : std::as_const(m_resources)) {
             ret += res->licenses();
         }
         kRemoveDuplicates(ret, [](const QJsonValueRef &val) -> QString {
@@ -174,25 +190,42 @@ public:
     QString longDescription() override
     {
         QStringList changes;
-        const auto resources = withoutDuplicates();
-        for (auto res : resources) {
-            const auto changelog = res->changelog();
+        auto resources = withoutDuplicates();
+
+        {
+            QCollator collator;
+            collator.setCaseSensitivity(Qt::CaseInsensitive);
+            const auto sortUpdateItems = [&collator](auto *a, auto *b) -> bool {
+                return collator(a->name(), b->name());
+            };
+            std::sort(resources.begin(), resources.end(), sortUpdateItems);
+        }
+
+        for (auto resource : std::as_const(resources)) {
+            const auto changelog = resource->changelog();
             if (changelog.isEmpty()) {
-                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>No release notes provided", res->packageName(), res->availableVersion());
+                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>No release notes provided", resource->packageName(), resource->availableVersion());
             } else {
                 changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>Release notes:<blockquote>%3</blockquote>",
-                                res->packageName(),
-                                res->availableVersion(),
+                                resource->packageName(),
+                                resource->availableVersion(),
                                 changelog);
             }
         }
-        changes.sort();
+
+        if (isDistroUpgrade()) {
+            changes.prepend(m_distroUpgrade->description());
+        }
         return changes.join(QString());
     }
     void fetchChangelog() override
     {
-        for (auto res : qAsConst(m_resources)) {
-            res->fetchUpdateDetails();
+        if (isDistroUpgrade()) {
+            return;
+        }
+
+        for (auto resource : std::as_const(m_resources)) {
+            resource->fetchUpdateDetails();
         }
         Q_EMIT changelogFetched({});
     }
@@ -222,8 +255,8 @@ public:
     QSet<QString> allPackageNames() const
     {
         QSet<QString> ret;
-        for (auto res : qAsConst(m_resources)) {
-            ret += kToSet(qobject_cast<PackageKitResource *>(res)->allPackageNames());
+        for (auto resource : std::as_const(m_resources)) {
+            ret += kToSet(qobject_cast<PackageKitResource *>(resource)->allPackageNames());
         }
         return ret;
     }
@@ -237,16 +270,17 @@ public:
     void setCandidates(const QSet<AbstractResource *> &candidates)
     {
         const auto toDisconnect = (m_resources - candidates);
-        for (auto res : toDisconnect) {
-            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
-            disconnect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
+        for (auto resource : toDisconnect) {
+            disconnect(resource, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            disconnect(resource, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
 
         const auto newCandidates = (candidates - m_resources);
         m_resources = candidates;
-        for (auto res : newCandidates) {
-            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
-            connect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
+
+        for (auto resource : newCandidates) {
+            connect(resource, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            connect(resource, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
     }
 
@@ -257,6 +291,27 @@ public:
         }
     }
 
+    void setDistroUpgrade(const AppStream::Release &release)
+    {
+        m_distroUpgrade = release;
+    }
+
+    void clearDistroUpgrade()
+    {
+        m_distroUpgrade = std::nullopt;
+        Q_EMIT m_backend->inlineMessageChanged({});
+    }
+
+    bool isDistroUpgrade() const
+    {
+        return m_distroUpgrade.has_value();
+    }
+
+    const AppStream::Release &getDistroUpgrade()
+    {
+        return m_distroUpgrade.value();
+    }
+
 Q_SIGNALS:
     void updateSizeChanged();
 
@@ -264,6 +319,7 @@ private:
     QSet<AbstractResource *> m_resources;
     PackageKitBackend *const m_backend;
     QTimer *m_updateSizeTimer;
+    std::optional<AppStream::Release> m_distroUpgrade;
 };
 
 PackageKitUpdater::PackageKitUpdater(PackageKitBackend *parent)
@@ -285,16 +341,17 @@ PackageKitUpdater::~PackageKitUpdater()
 
 void PackageKitUpdater::prepare()
 {
-    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+    auto offline = PackageKit::Daemon::global()->offline();
+    if (offline->updateTriggered() || offline->upgradeTriggered()) {
         m_toUpgrade.clear();
         m_allUpgradeable.clear();
-        enableNeedsReboot();
+        setNeedsReboot(true);
         return;
     }
 
     if (QFile::exists(QStringLiteral(PK_OFFLINE_RESULTS_FILENAME))) {
-        qDebug() << "Removed offline results file";
-        PackageKit::Daemon::global()->offline()->clearResults();
+        qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "Removed offline results file";
+        offline->clearResults();
     }
 
     Q_ASSERT(!m_transaction);
@@ -314,12 +371,12 @@ void PackageKitUpdater::prepare()
 
 void PackageKitUpdater::checkFreeSpace()
 {
-    auto j = KIO::fileSystemFreeSpace(QUrl::fromLocalFile("/usr"));
-    connect(j, &KIO::FileSystemFreeSpaceJob::result, this, [this](KIO::Job * /*job*/, KIO::filesize_t /*size*/, KIO::filesize_t available) {
-        if (available < updateSize()) {
+    auto job = KIO::fileSystemFreeSpace(QUrl::fromLocalFile(QStringLiteral("/usr")));
+    connect(job, &KJob::result, this, [this, job]() {
+        if (job->availableSize() < updateSize()) {
             setErrorMessage(i18nc("@info:status %1 is a formatted disk space string e.g. '240 MiB'",
                                   "Not enough space to perform the update; only %1 of space are available.",
-                                  KFormat().formatByteSize(available)));
+                                  KFormat().formatByteSize(job->availableSize())));
         }
     });
 }
@@ -327,9 +384,15 @@ void PackageKitUpdater::checkFreeSpace()
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesModified.clear();
-    auto pkgs = involvedPackages(m_toUpgrade).values();
-    pkgs.sort();
-    m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
+    if (m_toUpgrade.contains(m_upgrade) && m_upgrade->isDistroUpgrade()) {
+        const QString &upgradeVersion = m_upgrade->getDistroUpgrade().version();
+        m_transaction = PackageKit::Daemon::upgradeSystem(upgradeVersion, PackageKit::Transaction::UpgradeKind::UpgradeKindComplete, flags);
+        m_transaction->setHints(m_backend->globalHints() << QStringLiteral("cache-age=86400" /* 24*60*60 */));
+    } else {
+        auto pkgs = involvedPackages(m_toUpgrade).values();
+        pkgs.sort();
+        m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
+    }
     m_isCancelable = m_transaction->allowCancel();
     cancellableChanged();
 
@@ -340,17 +403,19 @@ void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFla
     connect(m_transaction.data(), &PackageKit::Transaction::eulaRequired, this, &PackageKitUpdater::eulaRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::repoSignatureRequired, this, &PackageKitUpdater::repoSignatureRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::allowCancelChanged, this, &PackageKitUpdater::cancellableChanged);
-    connect(m_transaction.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitUpdater::percentageChanged);
     connect(m_transaction.data(), &PackageKit::Transaction::itemProgress, this, &PackageKitUpdater::itemProgress);
     connect(m_transaction.data(), &PackageKit::Transaction::speedChanged, this, [this] {
         Q_EMIT downloadSpeedChanged(downloadSpeed());
     });
-    if (m_toUpgrade.contains(m_upgrade)) {
-        connect(m_transaction, &PackageKit::Transaction::percentageChanged, this, [this] {
-            if (m_transaction->status() == PackageKit::Transaction::StatusDownload) {
-                Q_EMIT resourceProgressed(m_upgrade, m_transaction->percentage(), Downloading);
-            }
-        });
+    if (!(flags & PackageKit::Transaction::TransactionFlagSimulate)) {
+        connect(m_transaction.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitUpdater::percentageChanged);
+        if (m_toUpgrade.contains(m_upgrade)) {
+            connect(m_transaction, &PackageKit::Transaction::percentageChanged, this, [this] {
+                if (m_transaction->status() == PackageKit::Transaction::StatusDownload) {
+                    Q_EMIT resourceProgressed(m_upgrade, m_transaction->percentage(), Downloading);
+                }
+            });
+        }
     }
 }
 
@@ -361,17 +426,17 @@ QSet<AbstractResource *> PackageKitUpdater::packagesForPackageId(const QSet<QStr
     });
 
     QSet<AbstractResource *> ret;
-    for (AbstractResource *res : qAsConst(m_allUpgradeable)) {
-        if (auto upgrade = dynamic_cast<SystemUpgrade *>(res)) {
-            if (packages.contains(upgrade->allPackageNames())) {
-                ret += upgrade;
-            }
-            continue;
+    for (auto resource : std::as_const(m_allUpgradeable)) {
+        QSet<QString> allPackageNames;
+
+        if (auto upgrade = qobject_cast<SystemUpgrade *>(resource)) {
+            allPackageNames = upgrade->allPackageNames();
+        } else if (auto pkResource = qobject_cast<PackageKitResource *>(resource)) {
+            allPackageNames = kToSet(pkResource->allPackageNames());
         }
 
-        PackageKitResource *pres = qobject_cast<PackageKitResource *>(res);
-        if (packages.contains(kToSet(pres->allPackageNames()))) {
-            ret.insert(res);
+        if (!allPackageNames.isEmpty() && packages.contains(allPackageNames)) {
+            ret.insert(resource);
         }
     }
 
@@ -382,16 +447,16 @@ QSet<QString> PackageKitUpdater::involvedPackages(const QSet<AbstractResource *>
 {
     QSet<QString> packageIds;
     packageIds.reserve(packages.size());
-    for (AbstractResource *res : packages) {
-        if (SystemUpgrade *upgrade = dynamic_cast<SystemUpgrade *>(res)) {
+    for (auto resource : packages) {
+        if (auto upgrade = qobject_cast<SystemUpgrade *>(resource)) {
             packageIds = involvedPackages(upgrade->resources());
             continue;
         }
 
-        PackageKitResource *app = qobject_cast<PackageKitResource *>(res);
-        const QSet<QString> ids = m_backend->upgradeablePackageId(app);
+        auto pkResource = qobject_cast<PackageKitResource *>(resource);
+        const QSet<QString> ids = m_backend->upgradeablePackageId(pkResource);
         if (ids.isEmpty()) {
-            qWarning() << "no upgradeablePackageId for" << app;
+            qWarning() << "no upgradeablePackageId for" << pkResource;
             continue;
         }
 
@@ -420,17 +485,18 @@ void PackageKitUpdater::processProceedFunction()
 
 void PackageKitUpdater::proceed()
 {
-    if (!m_proceedFunctions.isEmpty())
+    if (!m_proceedFunctions.isEmpty()) {
         processProceedFunction();
-    else if (useOfflineUpdates())
+    } else if (useOfflineUpdates()) {
         setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted | PackageKit::Transaction::TransactionFlagOnlyDownload);
-    else
+    } else {
         setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted);
+    }
 }
 
 bool PackageKitUpdater::useOfflineUpdates() const
 {
-    return m_useOfflineUpdates || qEnvironmentVariableIntValue("PK_OFFLINE_UPDATE");
+    return m_useOfflineUpdates || m_upgrade->isDistroUpgrade() || qEnvironmentVariableIntValue("PK_OFFLINE_UPDATE");
 }
 
 void PackageKitUpdater::setOfflineUpdates(bool use)
@@ -438,6 +504,21 @@ void PackageKitUpdater::setOfflineUpdates(bool use)
     m_useOfflineUpdates = use;
 }
 
+void PackageKitUpdater::setDistroUpgrade(const AppStream::Release &release)
+{
+    m_upgrade->setDistroUpgrade(release);
+}
+
+void PackageKitUpdater::clearDistroUpgrade()
+{
+    m_upgrade->clearDistroUpgrade();
+}
+
+bool PackageKitUpdater::isDistroUpgrade() const
+{
+    return m_upgrade->isDistroUpgrade();
+}
+
 void PackageKitUpdater::start()
 {
     Q_ASSERT(!isProgressing());
@@ -446,15 +527,16 @@ void PackageKitUpdater::start()
     setProgressing(true);
 
     if (useOfflineUpdates()) {
-        enableNeedsReboot();
+        setNeedsReboot(true);
     }
 }
 
 void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time*/)
 {
-    // qCDebug(LIBDISCOVER_BACKEND_LOG) << "update finished!" << exit << time;
-    if (!m_proceedFunctions.isEmpty())
+    // qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "update finished!" << exit << time;
+    if (!m_proceedFunctions.isEmpty()) {
         return;
+    }
     const bool cancel = exit == PackageKit::Transaction::ExitCancelled;
     const bool simulate = m_transaction->transactionFlags() & PackageKit::Transaction::TransactionFlagSimulate;
 
@@ -463,9 +545,19 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
 
     if (!cancel && simulate) {
         auto toremoveOrig = m_packagesModified.value(PackageKit::Transaction::InfoRemoving);
+#ifdef QPK_CHECK_VERSION
+#if QPK_CHECK_VERSION(1, 1, 2)
+        toremoveOrig += m_packagesModified.value(PackageKit::Transaction::InfoRemove);
+#endif
+#endif
         auto toremove = toremoveOrig;
         auto toinstall = QStringList() << m_packagesModified.value(PackageKit::Transaction::InfoInstalling)
                                        << m_packagesModified.value(PackageKit::Transaction::InfoUpdating);
+#ifdef QPK_CHECK_VERSION
+#if QPK_CHECK_VERSION(1, 1, 2)
+        toinstall << m_packagesModified.value(PackageKit::Transaction::InfoInstall);
+#endif
+#endif
 
         // some backends will treat upgrades as removal + install, which makes for terrible error messages.
         for (auto it = toremove.begin(), itEnd = toremove.end(); it != itEnd;) {
@@ -481,10 +573,10 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
             }
         };
 
-        if (PackageKit::Daemon::backendName() == "dnf") {
+        if (PackageKit::Daemon::backendName() == QLatin1String("dnf")) {
             // Fedora has some packages that it uninstalls then eventually creates on its own. No need to
             // notify about these.
-            toremove = kFilter<typeof(toremove)>(toremove, [](const QString &pkgid) {
+            toremove = kFilter<QStringList>(toremove, [](const QString &pkgid) {
                 return !PackageKit::Transaction::packageName(pkgid).startsWith(QLatin1String("kmod"));
             });
         }
@@ -492,11 +584,11 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
         if (!toremove.isEmpty()) {
             QStringList criticals;
             for (const auto &pkgid : std::as_const(toremove)) {
-                auto res = kFilter<QVector<AbstractResource *>>(m_backend->resourcesByPackageName(pkgid), [](AbstractResource *res) {
-                    return static_cast<PackageKitResource *>(res)->isCritical();
+                auto resources = kFilter<QVector<AbstractResource *>>(m_backend->resourcesByPackageName(pkgid), [](AbstractResource *resource) {
+                    return static_cast<PackageKitResource *>(resource)->isCritical();
                 });
-                criticals << kTransform<QStringList>(res, [](AbstractResource *a) {
-                    return a->name();
+                criticals << kTransform<QStringList>(resources, [](AbstractResource *resource) {
+                    return resource->name();
                 });
                 if (!criticals.isEmpty()) {
                     break;
@@ -512,7 +604,7 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
                 Q_EMIT distroErrorMessage(msg);
             } else {
                 Q_EMIT proceedRequest(
-                    i18n("Packages to remove"),
+                    i18n("Confirm Changes"),
                     i18n("The following packages will be removed by the update:<ul><li>%1</li></ul><br/>in order to install:<ul><li>%2</li></ul>",
                          PackageKitResource::joinPackages(toremove, QStringLiteral("</li><li>"), {}),
                          PackageKitResource::joinPackages(toinstall, QStringLiteral("</li><li>"), {})));
@@ -523,12 +615,35 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
         return;
     }
 
-    setProgressing(false);
-    m_backend->fetchUpdates();
-    fetchLastUpdateTime();
+    if (!m_toUpgrade.contains(m_upgrade) || !m_upgrade->isDistroUpgrade()) {
+        // If the distro-upgrade was prepared successfully, fetching updates now would invalidate it.
+        // If the distro-upgrade failed, fetching updates now would find nothing and we would propose to reboot.
+        setProgressing(false);
+        m_backend->fetchUpdates();
+        fetchLastUpdateTime();
+    } else if (exit != PackageKit::Transaction::ExitSuccess) {
+        Q_EMIT resourceProgressed(m_upgrade, 0, None);
+        setNeedsReboot(false);
+        setProgressing(false);
+    }
 
     if (useOfflineUpdates() && exit == PackageKit::Transaction::ExitSuccess) {
-        PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
+        if (m_upgrade->isDistroUpgrade()) {
+            QDBusPendingReply<void> reply = PackageKit::Daemon::global()->offline()->triggerUpgrade(PackageKit::Offline::ActionReboot);
+            // Call may fail because of authorization
+            reply.waitForFinished();
+            if (reply.isError()) {
+                Q_EMIT resourceProgressed(m_upgrade, 0, None);
+                setNeedsReboot(false);
+                setProgressing(false);
+                Q_EMIT passiveMessage(reply.error().message());
+                return;
+            }
+            m_backend->clear();
+            setProgressing(false);
+        } else {
+            PackageKit::Daemon::global()->offline()->trigger(m_offlineUpdateAction);
+        }
         enableReadyToReboot();
     }
 }
@@ -543,7 +658,11 @@ void PackageKitUpdater::cancellableChanged()
 
 void PackageKitUpdater::percentageChanged()
 {
-    const auto actualPercentage = percentageWithStatus(m_transaction->status(), m_transaction->percentage());
+    const int percentage = m_transaction->percentage();
+    if (percentage > 100) {
+        return;
+    }
+    const auto actualPercentage = useOfflineUpdates() ? percentage : percentageWithStatus(m_transaction->status(), percentage);
     if (actualPercentage >= 0 && m_percentage != actualPercentage) {
         m_percentage = actualPercentage;
         Q_EMIT progressChanged(m_percentage);
@@ -560,15 +679,15 @@ qreal PackageKitUpdater::progress() const
     return m_percentage;
 }
 
-void PackageKitUpdater::removeResources(const QList<AbstractResource *> &apps)
+void PackageKitUpdater::removeResources(const QList<AbstractResource *> &resources)
 {
-    const QSet<QString> pkgs = involvedPackages(kToSet(apps));
+    const QSet<QString> pkgs = involvedPackages(kToSet(resources));
     m_toUpgrade.subtract(packagesForPackageId(pkgs));
 }
 
-void PackageKitUpdater::addResources(const QList<AbstractResource *> &apps)
+void PackageKitUpdater::addResources(const QList<AbstractResource *> &resources)
 {
-    const QSet<QString> pkgs = involvedPackages(kToSet(apps));
+    const QSet<QString> pkgs = involvedPackages(kToSet(resources));
     m_toUpgrade.unite(packagesForPackageId(pkgs));
 }
 
@@ -577,9 +696,9 @@ QList<AbstractResource *> PackageKitUpdater::toUpdate() const
     return m_toUpgrade.values();
 }
 
-bool PackageKitUpdater::isMarked(AbstractResource *res) const
+bool PackageKitUpdater::isMarked(AbstractResource *resource) const
 {
-    return m_toUpgrade.contains(res);
+    return m_toUpgrade.contains(resource);
 }
 
 QDateTime PackageKitUpdater::lastUpdate() const
@@ -599,10 +718,11 @@ bool PackageKitUpdater::isProgressing() const
 
 void PackageKitUpdater::cancel()
 {
-    if (m_transaction)
+    if (m_transaction) {
         m_transaction->cancel();
-    else
+    } else {
         setProgressing(false);
+    }
 }
 
 void PackageKitUpdater::errorFound(PackageKit::Transaction::Error err, const QString &error)
@@ -624,7 +744,7 @@ void PackageKitUpdater::mediaChange(PackageKit::Transaction::MediaType media, co
 
 EulaHandling handleEula(const QString &eulaID, const QString &licenseAgreement)
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "EULA");
+    KConfigGroup group(KSharedConfig::openConfig(), u"EULA"_s);
     auto licenseGroup = group.group(eulaID);
     QCryptographicHash hash(QCryptographicHash::Sha256);
     hash.addData(licenseAgreement.toUtf8());
@@ -638,7 +758,7 @@ EulaHandling handleEula(const QString &eulaID, const QString &licenseAgreement)
         };
     } else {
         ret.proceedFunction = [eulaID, hashHex] {
-            KConfigGroup group(KSharedConfig::openConfig(), "EULA");
+            KConfigGroup group(KSharedConfig::openConfig(), u"EULA"_s);
             KConfigGroup licenseGroup = group.group(eulaID);
             licenseGroup.writeEntry<QByteArray>("Hash", hashHex);
             return PackageKit::Daemon::acceptEula(eulaID);
@@ -704,7 +824,7 @@ AbstractBackendUpdater::State toUpdateState(PackageKit::Transaction::Status t)
     case PackageKit::Transaction::StatusCancel:
         return AbstractBackendUpdater::Done;
     default:
-        qCDebug(LIBDISCOVER_BACKEND_LOG) << "unknown packagekit status" << t;
+        qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "unknown packagekit status" << t;
         return AbstractBackendUpdater::None;
     }
     Q_UNREACHABLE();
@@ -712,28 +832,28 @@ AbstractBackendUpdater::State toUpdateState(PackageKit::Transaction::Status t)
 
 void PackageKitUpdater::itemProgress(const QString &itemID, PackageKit::Transaction::Status status, uint percentage)
 {
-    const auto res = packagesForPackageId({itemID});
+    const auto resources = packagesForPackageId({itemID});
 
-    for (auto r : res) {
-        Q_EMIT resourceProgressed(r, percentage, toUpdateState(status));
+    for (auto resource : resources) {
+        Q_EMIT resourceProgressed(resource, percentage, toUpdateState(status));
     }
 }
 
 void PackageKitUpdater::fetchChangelog() const
 {
     QStringList pkgids;
-    for (AbstractResource *res : qAsConst(m_allUpgradeable)) {
-        if (auto upgrade = dynamic_cast<SystemUpgrade *>(res)) {
+    for (auto resource : std::as_const(m_allUpgradeable)) {
+        if (auto upgrade = qobject_cast<SystemUpgrade *>(resource)) {
             upgrade->fetchChangelog();
         } else {
-            pkgids += static_cast<PackageKitResource *>(res)->availablePackageId();
+            pkgids += static_cast<PackageKitResource *>(resource)->availablePackageId();
         }
     }
     Q_ASSERT(!pkgids.isEmpty());
 
-    PackageKit::Transaction *t = PackageKit::Daemon::getUpdatesDetails(pkgids);
-    connect(t, &PackageKit::Transaction::updateDetail, this, &PackageKitUpdater::updateDetail);
-    connect(t, &PackageKit::Transaction::errorCode, this, &PackageKitUpdater::errorFound);
+    PackageKit::Transaction *transaction = PackageKit::Daemon::getUpdatesDetails(pkgids);
+    connect(transaction, &PackageKit::Transaction::updateDetail, this, &PackageKitUpdater::updateDetail);
+    connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitUpdater::errorFound);
 }
 
 void PackageKitUpdater::updateDetail(const QString &packageID,
@@ -749,10 +869,10 @@ void PackageKitUpdater::updateDetail(const QString &packageID,
                                      const QDateTime &issued,
                                      const QDateTime &updated)
 {
-    const auto res = packagesForPackageId({packageID});
-    for (auto r : res) {
-        static_cast<PackageKitResource *>(r)
-            ->updateDetail(packageID, updates, obsoletes, vendorUrls, bugzillaUrls, cveUrls, restart, updateText, changelog, state, issued, updated);
+    const auto resources = packagesForPackageId({packageID});
+    for (auto resource : resources) {
+        auto pkResource = static_cast<PackageKitResource *>(resource);
+        pkResource->updateDetail(packageID, updates, obsoletes, vendorUrls, bugzillaUrls, cveUrls, restart, updateText, changelog, state, issued, updated);
     }
 }
 
@@ -786,17 +906,15 @@ double PackageKitUpdater::updateSize() const
 {
     double ret = 0.;
     QSet<QString> donePkgs;
-    for (AbstractResource *res : m_toUpgrade) {
-        if (auto upgrade = dynamic_cast<SystemUpgrade *>(res)) {
+    for (auto resource : std::as_const(m_toUpgrade)) {
+        if (auto upgrade = qobject_cast<SystemUpgrade *>(resource)) {
             ret += upgrade->size();
-            continue;
-        }
-
-        PackageKitResource *app = qobject_cast<PackageKitResource *>(res);
-        QString pkgname = app->packageName();
-        if (!donePkgs.contains(pkgname)) {
-            donePkgs.insert(pkgname);
-            ret += app->size();
+        } else if (auto pkResource = qobject_cast<PackageKitResource *>(resource)) {
+            QString pkgname = pkResource->packageName();
+            if (!donePkgs.contains(pkgname)) {
+                donePkgs.insert(pkgname);
+                ret += pkResource->size();
+            }
         }
     }
     return ret;
@@ -807,4 +925,13 @@ quint64 PackageKitUpdater::downloadSpeed() const
     return m_transaction ? m_transaction->speed() : 0;
 }
 
+void PackageKitUpdater::setOfflineUpdateAction(PackageKit::Offline::Action action)
+{
+    m_offlineUpdateAction = action;
+    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+        PackageKit::Daemon::global()->offline()->trigger(action);
+    }
+}
+
 #include "PackageKitUpdater.moc"
+#include "moc_PackageKitUpdater.cpp"
