@@ -14,12 +14,16 @@ OstreeFormat::OstreeFormat(Format format, const QString &source)
 
     switch (format) {
     // Using classic ostree image format
-    // Example: fedora:fedora/38/x86_64/kinoite
+    // Examples:
+    // fedora:fedora/38/x86_64/kinoite
+    // fedora/38/x86_64/kinoite
     case Format::Classic: {
         // Get remote and ref from the ostree source
-        auto split_ref = source.split(':');
+        auto split_ref = source.split(QLatin1Char(':'));
         if (split_ref.length() != 2) {
             // Unknown or invalid format
+            m_remote = QString();
+            m_branch = source;
             return;
         }
         m_remote = split_ref[0];
@@ -41,7 +45,7 @@ OstreeFormat::OstreeFormat(Format format, const QString &source)
     // ostree-image-signed:docker://<oci image>
     case Format::OCI: {
         // All early returns correspond to an unknown or invalid format
-        QStringList split = source.split(':');
+        QStringList split = source.split(QLatin1Char(':'));
         if ((split.length() < 2) || (split.length() > 5)) {
             return;
         }
@@ -118,7 +122,7 @@ bool OstreeFormat::parseTransport(QStringList *split)
     }
     // Special case for the 'docker://' transport
     if (split->first() == QLatin1String("docker")) {
-        m_transport += "docker://";
+        m_transport += QLatin1String("docker://");
         split->removeFirst();
         if (split->empty()) {
             return false;
@@ -192,3 +196,5 @@ QString OstreeFormat::transport() const
     }
     return m_transport;
 }
+
+#include "moc_OstreeFormat.cpp"
