@@ -14,12 +14,7 @@
 #include <resources/AbstractResourcesBackend.h>
 #include <resources/StandardBackendUpdater.h>
 
-#ifdef DISCOVER_USE_STABLE_APPSTREAM
-#include <AppStreamQt5/pool.h>
-#else
 #include <AppStreamQt/pool.h>
-#endif
-
 #include <QTimer>
 
 class DiscoverAction;
@@ -85,7 +80,7 @@ private:
     void setupTransaction(RpmOstreeTransaction::Operation op, QString arg = {});
 
     /* Called when a new major version is found to setup user facing actions */
-    void foundNewMajorVersion(const QString &newMajorVersion);
+    void foundNewMajorVersion(const AppStream::Release &release);
 
     /* Set to true once we've successfully registrered with rpm-ostree */
     bool m_registrered;
