@@ -6,8 +6,11 @@
 
 #include "BackendNotifierModule.h"
 
-BackendNotifierModule::BackendNotifierModule(QObject* parent)
+BackendNotifierModule::BackendNotifierModule(QObject *parent)
     : QObject(parent)
-{}
+{
+}
 
 BackendNotifierModule::~BackendNotifierModule() = default;
+
+#include "moc_BackendNotifierModule.cpp"
