@@ -22,6 +22,27 @@ class Category;
 class AbstractReviewsBackend;
 class AbstractBackendUpdater;
 
+struct StreamResult {
+    StreamResult(AbstractResource *resource = nullptr, uint sortScore = 0)
+        : resource(resource)
+        , sortScore(sortScore)
+    {
+    }
+
+    AbstractResource *resource = nullptr;
+    uint sortScore = 0;
+
+    bool operator==(const StreamResult &other) const
+    {
+        return resource == other.resource;
+    }
+};
+
+inline size_t qHash(const StreamResult &key, size_t seed = 0)
+{
+    return qHash(quintptr(key.resource), seed);
+}
+
 class DISCOVERCOMMON_EXPORT ResultsStream : public QObject
 {
     Q_OBJECT
@@ -29,13 +50,13 @@ public:
     ResultsStream(const QString &objectName);
 
     /// assumes all the information is in @p resources
-    ResultsStream(const QString &objectName, const QVector<AbstractResource *> &resources);
+    ResultsStream(const QString &objectName, const QVector<StreamResult> &resources);
     ~ResultsStream() override;
 
     void finish();
 
 Q_SIGNALS:
-    void resourcesFound(const QVector<AbstractResource *> &resources);
+    void resourcesFound(const QVector<StreamResult> &resources);
     void fetchMore();
 };
 
@@ -106,6 +127,8 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
     Q_PROPERTY(bool hasSecurityUpdates READ hasSecurityUpdates NOTIFY updatesCountChanged)
     Q_PROPERTY(bool isFetching READ isFetching NOTIFY fetchingChanged)
     Q_PROPERTY(bool hasApplications READ hasApplications CONSTANT)
+
+    Q_MOC_INCLUDE("ReviewsBackend/AbstractReviewsBackend.h")
 public:
     /**
      * Constructs an AbstractResourcesBackend
@@ -139,6 +162,7 @@ public:
 
         bool shouldFilter(AbstractResource *res) const;
         void filterJustInCase(QVector<AbstractResource *> &input) const;
+        void filterJustInCase(QVector<StreamResult> &input) const;
     };
 
     /**
@@ -179,7 +203,7 @@ public:
     /**
      * @returns the appstream ids that this backend extends
      */
-    virtual QStringList extends() const;
+    virtual bool extends(const QString &id) const;
 
     /** @returns the plugin's name */
     QString name() const;
@@ -209,6 +233,29 @@ public:
 
     virtual int fetchingUpdatesProgress() const;
 
+    /**
+     * @returns how much this backend should influence the global fetching progress.
+     * This is not a percentage.
+     */
+    virtual uint fetchingUpdatesProgressWeight() const;
+
+    enum AboutToAction { Reboot, PowerOff };
+
+    /**
+     * Notifies the backend to prepare for a certain power action.
+     * Currently only used in the PackageKit backend.
+     * @param action the action that will occur
+     */
+    virtual void aboutTo(AboutToAction action) { Q_UNUSED(action); };
+
+    /**
+     * @returns if the backend needs a reboot even when the PowerOff action is chosen,
+     * which would occur if the PowerOff happens after a reboot happens first
+     */
+    virtual bool needsRebootForPowerOffAction() const
+    {
+        return false;
+    }
 public Q_SLOTS:
     /**
      * This gets called when the backend should install an application.
@@ -272,11 +319,19 @@ Q_SIGNALS:
     void inlineMessageChanged(const QSharedPointer<InlineMessage> &inlineMessage);
     void fetchingUpdatesProgressChanged();
 
+    /**
+     * isValid will now return false
+     *
+     * A backend cannot become valid in its lifetime.
+     */
+    void invalidated();
+
 private:
     QString m_name;
 };
 
 DISCOVERCOMMON_EXPORT QDebug operator<<(QDebug dbg, const AbstractResourcesBackend::Filters &filters);
+DISCOVERCOMMON_EXPORT QDebug operator<<(QDebug dbg, const StreamResult &filters);
 
 /**
  * @internal Workaround because QPluginLoader enforces 1 instance per plugin
