@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef RESOURCESMODEL_H
-#define RESOURCESMODEL_H
+#pragma once
 
 #include <QSet>
 #include <QTimer>
@@ -32,14 +31,14 @@ Q_SIGNALS:
     void finished();
 
 private:
-    void addResults(const QVector<AbstractResource *> &res);
+    void addResults(const QVector<StreamResult> &res);
     void emitResults();
     void streamDestruction(QObject *obj);
     void resourceDestruction(QObject *obj);
     void clear();
 
     QSet<QObject *> m_streams;
-    QVector<AbstractResource *> m_results;
+    QVector<StreamResult> m_results;
     QTimer m_delayedEmission;
 };
 
@@ -74,11 +73,14 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QObject
     Q_PROPERTY(int updatesCount READ updatesCount NOTIFY updatesCountChanged)
     Q_PROPERTY(bool hasSecurityUpdates READ hasSecurityUpdates NOTIFY updatesCountChanged)
     Q_PROPERTY(bool isFetching READ isFetching NOTIFY fetchingChanged)
+    Q_PROPERTY(bool isInitializing READ isInitializing NOTIFY allInitialized)
     Q_PROPERTY(AbstractResourcesBackend *currentApplicationBackend READ currentApplicationBackend WRITE setCurrentApplicationBackend NOTIFY
                    currentApplicationBackendChanged)
     Q_PROPERTY(DiscoverAction *updateAction READ updateAction CONSTANT)
     Q_PROPERTY(int fetchingUpdatesProgress READ fetchingUpdatesProgress NOTIFY fetchingUpdatesProgressChanged)
     Q_PROPERTY(QString applicationSourceName READ applicationSourceName NOTIFY currentApplicationBackendChanged)
+    Q_PROPERTY(InlineMessage *inlineMessage READ inlineMessage NOTIFY inlineMessageChanged)
+    Q_PROPERTY(QString distroName READ distroName CONSTANT)
 public:
     /** This constructor should be only used by unit tests.
      *  @p backendName defines what backend will be loaded when the backend is constructed.
@@ -96,6 +98,7 @@ public:
 
     bool isBusy() const;
     bool isFetching() const;
+    bool isInitializing() const;
 
     Q_SCRIPTABLE bool isExtended(const QString &id);
 
@@ -116,8 +119,15 @@ public:
         return m_fetchingUpdatesProgress.m_value;
     }
 
+    QString distroName() const;
     Q_INVOKABLE QUrl distroBugReportUrl();
 
+    void setInlineMessage(const QSharedPointer<InlineMessage> &inlineMessage);
+    InlineMessage *inlineMessage() const
+    {
+        return m_inlineMessage.data();
+    }
+
 public Q_SLOTS:
     void installApplication(AbstractResource *app, const AddonList &addons);
     void installApplication(AbstractResource *app);
@@ -134,6 +144,8 @@ Q_SIGNALS:
     void passiveMessage(const QString &message);
     void currentApplicationBackendChanged(AbstractResourcesBackend *currentApplicationBackend);
     void fetchingUpdatesProgressChanged(int fetchingUpdatesProgress);
+    void inlineMessageChanged(const QSharedPointer<InlineMessage> &inlineMessage);
+    void switchToUpdates();
 
 private Q_SLOTS:
     void callerFetchingChanged();
@@ -142,24 +154,25 @@ private Q_SLOTS:
 
 private:
     ///@p initialize tells if all backends load will be triggered on construction
-    explicit ResourcesModel(QObject *parent = nullptr, bool load = true);
+    explicit ResourcesModel(QObject *parent = nullptr);
     void init(bool load);
-    void addResourcesBackend(AbstractResourcesBackend *backend);
+    void addResourcesBackends(const QList<AbstractResourcesBackend *> &backends);
+    bool addResourcesBackend(AbstractResourcesBackend *backend);
     void registerBackendByName(const QString &name);
     void initApplicationsBackend();
     void slotFetching();
 
     bool m_isFetching;
+    bool m_isInitializing = true;
     QVector<AbstractResourcesBackend *> m_backends;
-    int m_initializingBackends;
+    int m_initializingBackendsCount;
     DiscoverAction *m_updateAction = nullptr;
     AbstractResourcesBackend *m_currentApplicationBackend;
-    QTimer *m_allInitializedEmitter;
+    QTimer m_allInitializedEmitter;
 
     EmitWhenChanged<int> m_updatesCount;
     EmitWhenChanged<int> m_fetchingUpdatesProgress;
+    QSharedPointer<InlineMessage> m_inlineMessage;
 
     static ResourcesModel *s_self;
 };
-
-#endif // RESOURCESMODEL_H
