@@ -10,6 +10,7 @@
 #include "libdiscover_debug.h"
 #include <QMetaProperty>
 #include <cmath>
+#include <qnamespace.h>
 #include <utils.h>
 
 #include "ResourcesModel.h"
@@ -26,6 +27,7 @@ const QHash<int, QByteArray> ResourcesProxyModel::s_roles = {{NameRole, "name"},
                                                              {RatingPointsRole, "ratingPoints"},
                                                              {RatingCountRole, "ratingCount"},
                                                              {SortableRatingRole, "sortableRating"},
+                                                             {SearchRelevanceRole, "searchRelevance"},
                                                              {InstalledRole, "isInstalled"},
                                                              {ApplicationRole, "application"},
                                                              {OriginRole, "origin"},
@@ -40,11 +42,58 @@ const QHash<int, QByteArray> ResourcesProxyModel::s_roles = {{NameRole, "name"},
                                                              {SizeRole, "size"},
                                                              {ReleaseDateRole, "releaseDate"}};
 
+int levenshteinDistance(const QString &source, const QString &target)
+{
+    if (source == target) {
+        return 0;
+    }
+
+    // Do a case insensitive version of it
+    const QString &sourceUp = source.toUpper();
+    const QString &targetUp = target.toUpper();
+
+    if (sourceUp == targetUp) {
+        return 0;
+    }
+
+    const int sourceCount = sourceUp.size();
+    const int targetCount = targetUp.size();
+
+    if (sourceUp.isEmpty()) {
+        return targetCount;
+    }
+
+    if (targetUp.isEmpty()) {
+        return sourceCount;
+    }
+
+    if (sourceCount > targetCount) {
+        return levenshteinDistance(targetUp, sourceUp);
+    }
+
+    QVector<int> column;
+    column.fill(0, targetCount + 1);
+    QVector<int> previousColumn;
+    previousColumn.reserve(targetCount + 1);
+    for (int i = 0; i < targetCount + 1; i++) {
+        previousColumn.append(i);
+    }
+
+    for (int i = 0; i < sourceCount; i++) {
+        column[0] = i + 1;
+        for (int j = 0; j < targetCount; j++) {
+            column[j + 1] = std::min({1 + column.at(j), 1 + previousColumn.at(1 + j), previousColumn.at(j) + ((sourceUp.at(i) == targetUp.at(j)) ? 0 : 1)});
+        }
+        column.swap(previousColumn);
+    }
+
+    return previousColumn.at(targetCount);
+}
+
 ResourcesProxyModel::ResourcesProxyModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_sortRole(NameRole)
     , m_sortOrder(Qt::AscendingOrder)
-    , m_sortByRelevancy(false)
     , m_currentStream(nullptr)
 {
     // new QAbstractItemModelTester(this, this);
@@ -99,28 +148,22 @@ void ResourcesProxyModel::setSortOrder(Qt::SortOrder sortOrder)
 void ResourcesProxyModel::setSearch(const QString &_searchText)
 {
     // 1-character searches are painfully slow. >= 2 chars are fine, though
-    const QString searchText = _searchText.count() <= 1 ? QString() : _searchText;
+    const QString searchText = _searchText.size() <= 1 ? QString() : _searchText;
 
-    const bool diff = searchText != m_filters.search;
-
-    if (diff) {
+    if (m_filters.search != searchText) {
         m_filters.search = searchText;
-        if (m_sortByRelevancy == searchText.isEmpty()) {
-            m_sortByRelevancy = !searchText.isEmpty();
-            Q_EMIT sortByRelevancyChanged(m_sortByRelevancy);
-        }
         invalidateFilter();
         Q_EMIT searchChanged(m_filters.search);
     }
 }
 
-void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resources)
+void ResourcesProxyModel::removeDuplicates(QVector<StreamResult> &resources)
 {
-    const auto cab = ResourcesModel::global()->currentApplicationBackend();
+    const auto currentApplicationBackend = ResourcesModel::global()->currentApplicationBackend();
     QHash<QString, QString> aliases;
-    QHash<QString, QVector<AbstractResource *>::iterator> storedIds;
+    QHash<QString, QVector<StreamResult>::iterator> storedIds;
     for (auto it = m_displayedResources.begin(); it != m_displayedResources.end(); ++it) {
-        const auto appstreamid = (*it)->appstreamId();
+        const auto appstreamid = it->resource->appstreamId();
         if (appstreamid.isEmpty()) {
             continue;
         }
@@ -132,15 +175,15 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
             Q_UNREACHABLE();
         }
 
-        const auto alts = (*it)->alternativeAppstreamIds();
+        const auto alts = it->resource->alternativeAppstreamIds();
         for (const auto &alias : alts) {
             aliases[alias] = appstreamid;
         }
     }
 
-    QHash<QString, QVector<AbstractResource *>::iterator> ids;
+    QHash<QString, QVector<StreamResult>::iterator> ids;
     for (auto it = resources.begin(); it != resources.end();) {
-        const auto appstreamid = (*it)->appstreamId();
+        const auto appstreamid = it->resource->appstreamId();
         if (appstreamid.isEmpty()) {
             ++it;
             continue;
@@ -154,17 +197,19 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
         }
 
         if (at == storedIds.end()) {
-            const auto alts = (*it)->alternativeAppstreamIds();
+            const auto alts = it->resource->alternativeAppstreamIds();
             for (const auto &alt : alts) {
                 at = storedIds.find(alt);
-                if (at == storedIds.end())
+                if (at == storedIds.end()) {
                     break;
+                }
 
                 auto aliased = aliases.constFind(alt);
                 if (aliased != aliases.constEnd()) {
                     at = storedIds.find(aliased.value());
-                    if (at != storedIds.end())
+                    if (at != storedIds.end()) {
                         break;
+                    }
                 }
             }
         }
@@ -177,35 +222,37 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
                 }
             }
             if (at == ids.end()) {
-                const auto alts = (*it)->alternativeAppstreamIds();
+                const auto alts = it->resource->alternativeAppstreamIds();
                 for (const auto &alt : alts) {
                     at = ids.find(alt);
-                    if (at != ids.end())
+                    if (at != ids.end()) {
                         break;
+                    }
 
                     auto aliased = aliases.constFind(appstreamid);
                     if (aliased != aliases.constEnd()) {
                         at = ids.find(aliased.value());
-                        if (at != ids.end())
+                        if (at != ids.end()) {
                             break;
+                        }
                     }
                 }
             }
             if (at == ids.end()) {
                 ids[appstreamid] = it;
-                const auto alts = (*it)->alternativeAppstreamIds();
+                const auto alts = it->resource->alternativeAppstreamIds();
                 for (const auto &alias : alts) {
                     aliases[alias] = appstreamid;
                 }
                 ++it;
             } else {
-                if ((*it)->backend() == cab && (*it)->backend() != (**at)->backend()) {
+                if (it->resource->backend() == currentApplicationBackend && it->resource->backend() != (**at).resource->backend()) {
                     qSwap(*it, **at);
                 }
                 it = resources.erase(it);
             }
         } else {
-            if ((*it)->backend() == cab) {
+            if (it->resource->backend() == currentApplicationBackend) {
                 **at = *it;
                 auto pos = index(*at - m_displayedResources.begin(), 0);
                 Q_EMIT dataChanged(pos, pos);
@@ -215,35 +262,34 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
     }
 }
 
-void ResourcesProxyModel::addResources(const QVector<AbstractResource *> &_res)
+void ResourcesProxyModel::addResources(const QVector<StreamResult> &results)
 {
-    auto res = _res;
-    m_filters.filterJustInCase(res);
+    auto resultsCopy = results;
+    m_filters.filterJustInCase(resultsCopy);
 
-    if (res.isEmpty())
+    if (resultsCopy.isEmpty()) {
         return;
+    }
 
-    if (!m_sortByRelevancy)
-        std::sort(res.begin(), res.end(), [this](AbstractResource *res, AbstractResource *res2) {
-            return lessThan(res, res2);
-        });
+    std::sort(resultsCopy.begin(), resultsCopy.end(), [this](const auto &left, const auto &right) {
+        return orderedLessThan(left, right);
+    });
 
-    sortedInsertion(res);
+    sortedInsertion(resultsCopy);
     fetchSubcategories();
 }
 
 void ResourcesProxyModel::invalidateSorting()
 {
-    if (m_displayedResources.isEmpty())
+    if (m_displayedResources.isEmpty()) {
         return;
-
-    if (!m_sortByRelevancy) {
-        beginResetModel();
-        std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](AbstractResource *res, AbstractResource *res2) {
-            return lessThan(res, res2);
-        });
-        endResetModel();
     }
+
+    beginResetModel();
+    std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](const auto &left, const auto &right) {
+        return orderedLessThan(left, right);
+    });
+    endResetModel();
 }
 
 QString ResourcesProxyModel::lastSearch() const
@@ -253,12 +299,10 @@ QString ResourcesProxyModel::lastSearch() const
 
 void ResourcesProxyModel::setOriginFilter(const QString &origin)
 {
-    if (origin == m_filters.origin)
-        return;
-
-    m_filters.origin = origin;
-
-    invalidateFilter();
+    if (m_filters.origin != origin) {
+        m_filters.origin = origin;
+        invalidateFilter();
+    }
 }
 
 QString ResourcesProxyModel::originFilter() const
@@ -271,54 +315,53 @@ QString ResourcesProxyModel::filteredCategoryName() const
     return m_categoryName;
 }
 
-void ResourcesProxyModel::setFilteredCategoryName(const QString &cat)
+void ResourcesProxyModel::setFilteredCategoryName(const QString &categoryName)
 {
-    if (cat == m_categoryName)
+    if (categoryName == m_categoryName) {
         return;
+    }
 
-    m_categoryName = cat;
+    m_categoryName = categoryName;
 
-    auto category = CategoryModel::global()->findCategoryByName(cat);
-    if (category) {
+    if (auto category = CategoryModel::global()->findCategoryByName(categoryName)) {
         setFiltersFromCategory(category);
     } else {
         qDebug() << "looking up wrong category or too early" << m_categoryName;
-        auto f = [this, cat] {
-            auto category = CategoryModel::global()->findCategoryByName(cat);
+        auto f = [this, categoryName] {
+            auto category = CategoryModel::global()->findCategoryByName(categoryName);
             setFiltersFromCategory(category);
         };
-        auto one = new OneTimeAction(f, this);
-        connect(CategoryModel::global(), &CategoryModel::rootCategoriesChanged, one, &OneTimeAction::trigger);
+        auto action = new OneTimeAction(f, this);
+        connect(CategoryModel::global(), &CategoryModel::rootCategoriesChanged, action, &OneTimeAction::trigger);
     }
 }
 
 void ResourcesProxyModel::setFiltersFromCategory(Category *category)
 {
-    if (category == m_filters.category)
-        return;
-
-    m_filters.category = category;
-    invalidateFilter();
-    Q_EMIT categoryChanged();
+    if (m_filters.category != category) {
+        m_filters.category = category;
+        invalidateFilter();
+        Q_EMIT categoryChanged();
+    }
 }
 
 void ResourcesProxyModel::fetchSubcategories()
 {
-    auto cats = kToSet(m_filters.category ? m_filters.category->subCategories() : CategoryModel::global()->rootCategories());
+    auto categories = kToSet(m_filters.category ? m_filters.category->subCategories() : CategoryModel::global()->rootCategories());
 
     const int count = rowCount();
     QSet<Category *> done;
-    for (int i = 0; i < count && !cats.isEmpty(); ++i) {
-        AbstractResource *res = m_displayedResources[i];
-        const auto found = res->categoryObjects(kSetToVector(cats));
+    for (int i = 0; i < count && !categories.isEmpty(); ++i) {
+        const auto resource = m_displayedResources[i].resource;
+        const auto found = resource->categoryObjects(kSetToVector(categories));
         done.unite(found);
-        cats.subtract(found);
+        categories.subtract(found);
     }
 
-    const QVariantList ret = kTransform<QVariantList>(done, [](Category *cat) {
-        return QVariant::fromValue<QObject *>(cat);
+    const QVariantList ret = kTransform<QVariantList>(done, [](Category *category) {
+        return QVariant::fromValue<QObject *>(category);
     });
-    if (ret != m_subcategories) {
+    if (m_subcategories != ret) {
         m_subcategories = ret;
         Q_EMIT subcategoriesChanged(m_subcategories);
     }
@@ -345,7 +388,7 @@ void ResourcesProxyModel::invalidateFilter()
     }
 
     m_currentStream = m_filters.backend ? m_filters.backend->search(m_filters) : ResourcesModel::global()->search(m_filters);
-    Q_EMIT busyChanged(true);
+    Q_EMIT busyChanged();
 
     if (!m_displayedResources.isEmpty()) {
         beginResetModel();
@@ -356,7 +399,7 @@ void ResourcesProxyModel::invalidateFilter()
     connect(m_currentStream, &ResultsStream::resourcesFound, this, &ResourcesProxyModel::addResources);
     connect(m_currentStream, &ResultsStream::destroyed, this, [this]() {
         m_currentStream = nullptr;
-        Q_EMIT busyChanged(false);
+        Q_EMIT busyChanged();
     });
 }
 
@@ -365,38 +408,38 @@ int ResourcesProxyModel::rowCount(const QModelIndex &parent) const
     return parent.isValid() ? 0 : m_displayedResources.count();
 }
 
-bool ResourcesProxyModel::lessThan(AbstractResource *leftPackage, AbstractResource *rightPackage) const
+// This comparator takes m_sortRole and m_sortOrder into account. It has a
+// fallback mechanism to use secondary sort role and order.
+bool ResourcesProxyModel::orderedLessThan(const StreamResult &left, const StreamResult &right) const
 {
-    auto role = m_sortRole;
-    Qt::SortOrder order = m_sortOrder;
-    QVariant leftValue;
-    QVariant rightValue;
-    // if we're comparing two equal values, we want the model sorted by application name
-    if (role != NameRole) {
-        leftValue = roleToValue(leftPackage, role);
-        rightValue = roleToValue(rightPackage, role);
+    // (role, order) pair
+    using SortCombination = std::pair<Roles, Qt::SortOrder>;
+    const std::array<SortCombination, 2> sortFallbackChain = {{
+        {m_sortRole, m_sortOrder},
+        {NameRole, Qt::AscendingOrder},
+    }};
+
+    for (const auto &[role, order] : sortFallbackChain) {
+        QVariant leftValue = roleToOrderedValue(left, role);
+        QVariant rightValue = roleToOrderedValue(right, role);
 
         if (leftValue == rightValue) {
-            role = NameRole;
-            order = Qt::AscendingOrder;
+            continue;
         }
+
+        const auto result = QVariant::compare(leftValue, rightValue);
+
+        // Should not happen, but it's better to skip than assert
+        if (!(result == QPartialOrdering::Less || result == QPartialOrdering::Greater)) {
+            continue;
+        }
+
+        // Yes, there is a shorter but incomprehensible way of rewriting this
+        return (order == Qt::AscendingOrder) ? (result == QPartialOrdering::Less) : (result == QPartialOrdering::Greater);
     }
 
-    bool ret;
-    if (role == NameRole) {
-        ret = leftPackage->nameSortKey().compare(rightPackage->nameSortKey()) < 0;
-    } else if (role == CanUpgrade) {
-        ret = leftValue.toBool();
-    } else {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        ret = leftValue < rightValue;
-#else
-        const auto order = QVariant::compare(leftValue, rightValue);
-        Q_ASSERT(order != QPartialOrdering::Unordered);
-        return order == QPartialOrdering::Less;
-#endif
-    }
-    return ret != (order != Qt::AscendingOrder);
+    // They compared equal, so it is definitely not a "less than" relation
+    return false;
 }
 
 Category *ResourcesProxyModel::filteredCategory() const
@@ -496,62 +539,99 @@ QVariant ResourcesProxyModel::data(const QModelIndex &index, int role) const
     if (!index.isValid()) {
         return QVariant();
     }
-
-    AbstractResource *const resource = m_displayedResources[index.row()];
-    return roleToValue(resource, role);
+    const auto result = m_displayedResources[index.row()];
+    return roleToValue(result, role);
 }
 
-QVariant ResourcesProxyModel::roleToValue(AbstractResource *resource, int role) const
+QHash<int, int> ResourcesProxyModel::createRoleToProperty()
 {
+    QHash<int, int> ret;
+    static const QMetaObject *m = &AbstractResource::staticMetaObject;
+    for (const auto &[role, roleText] : ResourcesProxyModel::s_roles.asKeyValueRange()) {
+        int propidx = roleText.isEmpty() ? -1 : m->indexOfProperty(roleText.constData());
+        ret.insert(role, propidx);
+    }
+    return ret;
+}
+
+QVariant ResourcesProxyModel::roleToValue(const StreamResult &result, int role) const
+{
+    const auto resource = result.resource;
     switch (role) {
     case ApplicationRole:
         return QVariant::fromValue<QObject *>(resource);
-    case RatingPointsRole:
-    case RatingRole:
-    case RatingCountRole:
+    case RatingPointsRole: {
+        const auto rating = resource->rating();
+        return rating.ratingPoints();
+    }
+    case RatingRole: {
+        const auto rating = resource->rating();
+        return rating.rating();
+    }
+    case RatingCountRole: {
+        const auto rating = resource->rating();
+        return rating.ratingCount();
+    }
     case SortableRatingRole: {
-        Rating *const rating = resource->rating();
-        const int idx = Rating::staticMetaObject.indexOfProperty(roleNames().value(role).constData());
-        Q_ASSERT(idx >= 0);
-        auto prop = Rating::staticMetaObject.property(idx);
-        if (rating) {
-            return prop.readOnGadget(rating);
-        } else {
-            QVariant val(0);
-            val.convert(prop.type());
-            return val;
+        const auto rating = resource->rating();
+        return rating.sortableRating();
+    }
+    case SearchRelevanceRole: {
+        qreal rating = roleToValue(result, SortableRatingRole).value<qreal>();
+
+        qreal reverseDistance = 0;
+        for (const QString &word : resource->name().split(QLatin1Char(' '))) {
+            const qreal maxLength = std::max(word.length(), m_filters.search.length());
+            reverseDistance =
+                std::max(reverseDistance, (maxLength - std::min(reverseDistance, qreal(levenshteinDistance(word, m_filters.search)))) / maxLength * 10.0);
         }
+
+        qreal exactMatch = 0.0;
+        if (resource->name().toUpper() == m_filters.search.toUpper()) {
+            exactMatch = 10.0;
+        } else if (resource->name().contains(m_filters.search, Qt::CaseInsensitive)) {
+            exactMatch = 5.0;
+        }
+        return qreal(result.sortScore) / 100 + rating + reverseDistance + exactMatch;
     }
     case Qt::DecorationRole:
     case Qt::DisplayRole:
     case Qt::StatusTipRole:
     case Qt::ToolTipRole:
-        return QVariant();
+        return {};
     default: {
-        QByteArray roleText = roleNames().value(role);
-        if (Q_UNLIKELY(roleText.isEmpty())) {
-            qCDebug(LIBDISCOVER_LOG) << "unsupported role" << role;
-            return {};
-        }
-        static const QMetaObject *m = &AbstractResource::staticMetaObject;
-        int propidx = roleText.isEmpty() ? -1 : m->indexOfProperty(roleText.constData());
-
+        static QHash<int, int> s_roleToProperty = createRoleToProperty();
+        const int propidx = s_roleToProperty[role];
         if (Q_UNLIKELY(propidx < 0)) {
-            qCWarning(LIBDISCOVER_LOG) << "unknown role:" << role << roleText;
-            return QVariant();
-        } else
-            return m->property(propidx).read(resource);
+            qCWarning(LIBDISCOVER_LOG) << "unknown role:" << role << roleNames()[role];
+            return {};
+        } else {
+            return AbstractResource::staticMetaObject.property(propidx).read(resource);
+        }
     }
     }
 }
 
-bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
+// Wraps roleToValue with additional features for sorting/comparison.
+QVariant ResourcesProxyModel::roleToOrderedValue(const StreamResult &result, int role) const
 {
-    auto last = resources.constFirst();
-    for (auto it = resources.constBegin() + 1, itEnd = resources.constEnd(); it != itEnd; ++it) {
-        auto v1 = roleToValue(last, m_sortRole), v2 = roleToValue(*it, m_sortRole);
-        if (!lessThan(last, *it) && v1 != v2) {
-            qDebug() << "faulty sort" << last->name() << (*it)->name() << last << (*it);
+    AbstractResource *resource = result.resource;
+    switch (role) {
+    case NameRole:
+        return QVariant::fromValue(resource->nameSortKey());
+    default:
+        return roleToValue(result, role);
+    }
+}
+
+bool ResourcesProxyModel::isSorted(const QVector<StreamResult> &results)
+{
+    auto last = results.constFirst();
+    for (auto it = results.constBegin() + 1, itEnd = results.constEnd(); it != itEnd; ++it) {
+        auto v1 = roleToValue(last, m_sortRole);
+        auto v2 = roleToValue(*it, m_sortRole);
+        if (!orderedLessThan(last, *it) && v1 != v2) {
+            qCDebug(LIBDISCOVER_LOG) << "Faulty sort" << last.resource->name() << (*it).resource->name() << last.resource << (*it).resource;
             return false;
         }
         last = *it;
@@ -559,77 +639,81 @@ bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
     return true;
 }
 
-void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_res)
+void ResourcesProxyModel::sortedInsertion(const QVector<StreamResult> &results)
 {
-    Q_ASSERT(_res.size() == QSet(_res.constBegin(), _res.constEnd()).size());
+    Q_ASSERT(results.size() == QSet(results.constBegin(), results.constEnd()).size());
 
-    auto resources = _res;
-    Q_ASSERT(!resources.isEmpty());
+    auto resultsCopy = results;
+    Q_ASSERT(!resultsCopy.isEmpty());
 
     if (!m_filters.allBackends) {
-        removeDuplicates(resources);
-        if (resources.isEmpty())
+        removeDuplicates(resultsCopy);
+        if (resultsCopy.isEmpty()) {
             return;
+        }
     }
 
-    if (m_sortByRelevancy || m_displayedResources.isEmpty()) {
-        // Q_ASSERT(m_sortByRelevancy || isSorted(resources));
+    if (m_displayedResources.isEmpty()) {
         int rows = rowCount();
-        beginInsertRows({}, rows, rows + resources.count() - 1);
-        m_displayedResources += resources;
+        beginInsertRows({}, rows, rows + resultsCopy.count() - 1);
+        m_displayedResources += resultsCopy;
         endInsertRows();
         return;
     }
 
-    for (auto resource : qAsConst(resources)) {
-        const auto finder = [this](AbstractResource *resource, AbstractResource *res) {
-            return lessThan(resource, res);
+    for (const auto &result : std::as_const(resultsCopy)) {
+        const auto finder = [this](const StreamResult &left, const StreamResult &right) {
+            return orderedLessThan(left, right);
         };
-        const auto it = std::upper_bound(m_displayedResources.constBegin(), m_displayedResources.constEnd(), resource, finder);
+        const auto it = std::upper_bound(m_displayedResources.constBegin(), m_displayedResources.constEnd(), result, finder);
+        const auto prev = it - 1;
         const auto newIdx = it == m_displayedResources.constEnd() ? m_displayedResources.count() : (it - m_displayedResources.constBegin());
 
-        if ((it - 1) != m_displayedResources.constEnd() && *(it - 1) == resource)
+        if (prev != m_displayedResources.constEnd() && prev->resource == result.resource) {
             continue;
+        }
 
         beginInsertRows({}, newIdx, newIdx);
-        m_displayedResources.insert(newIdx, resource);
+        m_displayedResources.insert(newIdx, result);
         endInsertRows();
-        // Q_ASSERT(isSorted(resources));
+        // Q_ASSERT(isSorted(resultsCopy));
     }
 }
 
 void ResourcesProxyModel::refreshResource(AbstractResource *resource, const QVector<QByteArray> &properties)
 {
-    const auto residx = m_displayedResources.indexOf(resource);
-    if (residx < 0) {
+    const auto row = indexOf(resource);
+    if (row < 0) {
         return;
     }
 
     if (!m_filters.shouldFilter(resource)) {
-        beginRemoveRows({}, residx, residx);
-        m_displayedResources.removeAt(residx);
+        beginRemoveRows({}, row, row);
+        m_displayedResources.removeAt(row);
         endRemoveRows();
         return;
     }
 
-    const QModelIndex idx = index(residx, 0);
+    const QModelIndex idx = index(row, 0);
     Q_ASSERT(idx.isValid());
     const auto roles = propertiesToRoles(properties);
-    if (!m_sortByRelevancy && roles.contains(m_sortRole)) {
-        beginRemoveRows({}, residx, residx);
-        m_displayedResources.removeAt(residx);
+    if (roles.contains(m_sortRole)) {
+        beginRemoveRows({}, row, row);
+        m_displayedResources.removeAt(row);
         endRemoveRows();
 
-        sortedInsertion({resource});
-    } else
+        sortedInsertion({{resource, 0}});
+    } else {
         Q_EMIT dataChanged(idx, idx, roles);
+    }
 }
 
 void ResourcesProxyModel::removeResource(AbstractResource *resource)
 {
-    const auto residx = m_displayedResources.indexOf(resource);
-    if (residx < 0)
+    const auto residx = indexOf(resource);
+    if (residx < 0) {
         return;
+    }
     beginRemoveRows({}, residx, residx);
     m_displayedResources.removeAt(residx);
     endRemoveRows();
@@ -643,11 +727,14 @@ void ResourcesProxyModel::refreshBackend(AbstractResourcesBackend *backend, cons
     bool found = false;
 
     for (int i = 0; i < count; ++i) {
-        if (backend != m_displayedResources[i]->backend())
+        if (backend != m_displayedResources[i].resource->backend()) {
             continue;
+        }
 
         int j = i + 1;
-        for (; j < count && backend == m_displayedResources[j]->backend(); ++j) { }
+        while (j < count && backend == m_displayedResources[j].resource->backend()) {
+            j++;
+        }
 
         Q_EMIT dataChanged(index(i, 0), index(j - 1, 0), roles);
         i = j;
@@ -661,21 +748,22 @@ void ResourcesProxyModel::refreshBackend(AbstractResourcesBackend *backend, cons
 
 QVector<int> ResourcesProxyModel::propertiesToRoles(const QVector<QByteArray> &properties) const
 {
-    QVector<int> roles = kTransform<QVector<int>>(properties, [this](const QByteArray &arr) {
-        return roleNames().key(arr, -1);
+    return kFilterTransform<QVector<int>>(properties, [this](const QByteArray &property) {
+        const auto role = roleNames().key(property, -1);
+        return role == -1 ? std::nullopt : std::optional(role);
     });
-    roles.removeAll(-1);
-    return roles;
 }
 
-int ResourcesProxyModel::indexOf(AbstractResource *res)
+int ResourcesProxyModel::indexOf(AbstractResource *resource)
 {
-    return m_displayedResources.indexOf(res);
+    return kIndexOf(m_displayedResources, [resource](const StreamResult &result) {
+        return result.resource == resource;
+    });
 }
 
 AbstractResource *ResourcesProxyModel::resourceAt(int row) const
 {
-    return m_displayedResources[row];
+    return m_displayedResources[row].resource;
 }
 
 bool ResourcesProxyModel::canFetchMore(const QModelIndex &parent) const
@@ -687,14 +775,9 @@ bool ResourcesProxyModel::canFetchMore(const QModelIndex &parent) const
 void ResourcesProxyModel::fetchMore(const QModelIndex &parent)
 {
     Q_ASSERT(!parent.isValid());
-    if (!m_currentStream)
-        return;
-    Q_EMIT m_currentStream->fetchMore();
-}
-
-bool ResourcesProxyModel::sortByRelevancy() const
-{
-    return m_sortByRelevancy;
+    if (m_currentStream) {
+        Q_EMIT m_currentStream->fetchMore();
+    }
 }
 
 ResourcesCount ResourcesProxyModel::count() const
@@ -737,3 +820,5 @@ ResourcesCount::ResourcesCount(int number, const QString &string)
     , m_string(string)
 {
 }
+
+#include "moc_ResourcesProxyModel.cpp"
