@@ -19,6 +19,8 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
+using namespace Qt::StringLiterals;
+
 class UpdateTransaction : public Transaction
 {
     Q_OBJECT
@@ -28,7 +30,7 @@ public:
         , m_allUpdaters(updaters)
     {
         bool cancelable = false;
-        for (auto updater : qAsConst(m_allUpdaters)) {
+        for (auto updater : std::as_const(m_allUpdaters)) {
             connect(updater, &AbstractBackendUpdater::progressingChanged, this, &UpdateTransaction::slotProgressingChanged);
             connect(updater, &AbstractBackendUpdater::downloadSpeedChanged, this, &UpdateTransaction::slotDownloadSpeedChanged);
             connect(updater, &AbstractBackendUpdater::progressChanged, this, &UpdateTransaction::slotUpdateProgress);
@@ -67,7 +69,7 @@ public:
     bool isProgressing() const
     {
         bool progressing = false;
-        for (AbstractBackendUpdater *upd : qAsConst(m_allUpdaters)) {
+        for (AbstractBackendUpdater *upd : std::as_const(m_allUpdaters)) {
             progressing |= upd->isProgressing();
         }
         return progressing;
@@ -85,7 +87,7 @@ public:
     void slotUpdateProgress()
     {
         qreal total = 0;
-        for (AbstractBackendUpdater *updater : qAsConst(m_allUpdaters)) {
+        for (AbstractBackendUpdater *updater : std::as_const(m_allUpdaters)) {
             total += updater->progress();
         }
         setProgress(total / m_allUpdaters.count());
@@ -94,7 +96,7 @@ public:
     void slotDownloadSpeedChanged()
     {
         quint64 total = 0;
-        for (AbstractBackendUpdater *updater : qAsConst(m_allUpdaters)) {
+        for (AbstractBackendUpdater *updater : std::as_const(m_allUpdaters)) {
             total += updater->downloadSpeed();
         }
         setDownloadSpeed(total);
@@ -151,13 +153,13 @@ void ResourcesUpdatesModel::init()
     // To enable from command line use:
     // kwriteconfig5 --file discoverrc --group Software --key UseOfflineUpdates true
     auto sharedConfig = KSharedConfig::openConfig();
-    KConfigGroup group(sharedConfig, "Software");
+    KConfigGroup group(sharedConfig, u"Software"_s);
     m_offlineUpdates = group.readEntry<bool>("UseOfflineUpdates", false);
 
     KConfigWatcher::Ptr watcher = KConfigWatcher::create(sharedConfig);
     connect(watcher.data(), &KConfigWatcher::configChanged, this, [this](const KConfigGroup &group, const QByteArrayList &names) {
         // Ensure it is for the right file
-        if (!names.contains("UseOfflineUpdates") || group.name() != "Software") {
+        if (!names.contains("UseOfflineUpdates") || group.name() != QLatin1String("Software")) {
             return;
         }
 
@@ -204,7 +206,7 @@ void ResourcesUpdatesModel::prepare()
         return;
     }
 
-    for (AbstractBackendUpdater *upd : qAsConst(m_updaters)) {
+    for (AbstractBackendUpdater *upd : std::as_const(m_updaters)) {
         upd->setOfflineUpdates(m_offlineUpdates);
         upd->prepare();
     }
@@ -248,7 +250,7 @@ bool ResourcesUpdatesModel::isProgressing() const
 QList<AbstractResource *> ResourcesUpdatesModel::toUpdate() const
 {
     QList<AbstractResource *> ret;
-    for (AbstractBackendUpdater *upd : qAsConst(m_updaters)) {
+    for (AbstractBackendUpdater *upd : std::as_const(m_updaters)) {
         ret += upd->toUpdate();
     }
     return ret;
@@ -281,7 +283,7 @@ void ResourcesUpdatesModel::removeResources(const QList<AbstractResource *> &res
 QDateTime ResourcesUpdatesModel::lastUpdate() const
 {
     QDateTime ret;
-    for (AbstractBackendUpdater *upd : qAsConst(m_updaters)) {
+    for (AbstractBackendUpdater *upd : std::as_const(m_updaters)) {
         QDateTime current = upd->lastUpdate();
         if (!ret.isValid() || (current.isValid() && current > ret)) {
             ret = current;
@@ -358,3 +360,4 @@ QStringList ResourcesUpdatesModel::errorMessages() const
 }
 
 #include "ResourcesUpdatesModel.moc"
+#include "moc_ResourcesUpdatesModel.cpp"
